@@ -1,7 +1,7 @@
 /*
  *	Blade DLL Interface for LAME.
  *
- *	Copyright (c) 1999 - 2001 A.L. Faber
+ *	Copyright (c) 1999 - 2002 A.L. Faber
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,28 +25,34 @@
 #include <assert.h>
 
 #include "lame.h"
+#include "config.h"
+// sample_t should not be needed by any libmp3lame wrapper:
+//#include "machine.h" /* for sample_t type */
+
+
+#define         Min(A, B)       ((A) < (B) ? (A) : (B))
+#define         Max(A, B)       ((A) > (B) ? (A) : (B))
 
 #define _RELEASEDEBUG 0
 
 // lame_enc DLL version number
 const int MAJORVERSION = 1;
-const int MINORVERSION = 26;
+const int MINORVERSION = 32;
 
 
 // Local variables
 static DWORD				dwSampleBufferSize=0;
 static HANDLE				gs_hModule=NULL;
 static BOOL					gs_bLogFile=FALSE;
-static lame_global_flags*	gfp = NULL;
+static lame_global_flags*	gfp_save = NULL;
 
 // Local function prototypes
-static void dump_config( );
+static void dump_config( 	lame_global_flags*	gfp );
 static void DebugPrintf( const char* pzFormat, ... );
 static void DispErr( LPSTR strErr );
 static void PresetOptions( lame_global_flags *gfp, LONG myPreset );
 
 
-
 static void DebugPrintf(const char* pzFormat, ...)
 {
     char	szBuffer[1024]={'\0',};
@@ -54,12 +60,14 @@ static void DebugPrintf(const char* pzFormat, ...)
     va_list ap;
 
 	// Get the full module (DLL) file name
-	GetModuleFileName(gs_hModule,szFileName,sizeof(szFileName));
+	GetModuleFileName(	gs_hModule, 
+						szFileName,
+						sizeof( szFileName ) );
 
 	// change file name extention
-	szFileName[strlen(szFileName)-3]='t';
-	szFileName[strlen(szFileName)-2]='x';
-	szFileName[strlen(szFileName)-1]='t';
+	szFileName[ strlen(szFileName) - 3 ] = 't';
+	szFileName[ strlen(szFileName) - 2 ] = 'x';
+	szFileName[ strlen(szFileName) - 1 ] = 't';
 
 	// start at beginning of the list
 	va_start(ap, pzFormat);
@@ -87,7 +95,7 @@ static void DebugPrintf(const char* pzFormat, ...)
     }
 
 #if defined _DEBUG || defined _RELEASEDEBUG
-    OutputDebugString(szBuffer);
+    OutputDebugString( szBuffer );
 #endif
 
 	va_end(ap);
@@ -98,162 +106,130 @@ static void PresetOptions( lame_global_flags *gfp, LONG myPreset )
 {
 	switch (myPreset)
 	{
-		case LQP_NOPRESET:
-		case LQP_NORMAL_QUALITY:
+/*-1*/		case LQP_NOPRESET:
 			break;
-		break;
-		case LQP_LOW_QUALITY:
-			lame_set_quality( gfp, 9 );
+
+/*0*/		case LQP_NORMAL_QUALITY:
+				lame_set_quality( gfp, 5 );
 			break;
-		case LQP_HIGH_QUALITY:
-			lame_set_quality( gfp, 2 );
+
+/*1*/		case LQP_LOW_QUALITY:
+				lame_set_quality( gfp, 9 );
 			break;
-		case LQP_VERYHIGH_QUALITY:
-			lame_set_quality( gfp, 0 );
+
+/*2*/		case LQP_HIGH_QUALITY:
+				lame_set_quality( gfp, 2 );
 			break;
-		case LQP_VOICE_QUALITY:		// --voice flag for experimental voice mode
-            lame_set_lowpassfreq( gfp, 12000 );
-            lame_set_VBR_max_bitrate_kbps( gfp, 160 );
-			lame_set_no_short_blocks( gfp, 1 );
-		break;
-		case LQP_R3MIX_QUALITY:
 
-            lame_set_exp_nspsytune( gfp, lame_get_exp_nspsytune( gfp ) | 1); /*nspsytune*/
+/*3*/		case LQP_VOICE_QUALITY:				// --voice flag for experimental voice mode
+				lame_set_mode( gfp, MONO );
+		        lame_set_preset( gfp, 56);
+			break;
 
-            (void) lame_set_scale( gfp, 0.98f ); /* --scale 0.98*/
+/*4*/		case LQP_R3MIX:					// --R3MIX
+		        lame_set_preset( gfp, R3MIX);
+			break;
 
-            lame_set_exp_nspsytune(gfp, lame_get_exp_nspsytune(gfp) | (8 << 20));
+/*5*/		case LQP_VERYHIGH_QUALITY:
+				lame_set_quality( gfp, 0 );
+			break;
 
-            lame_set_VBR( gfp,vbr_mtrh ); 
-            lame_set_VBR_q( gfp, 1 );
-            lame_set_quality( gfp, 2 );
-            lame_set_lowpassfreq( gfp, 19500 );
-            lame_set_mode( gfp, JOINT_STEREO );
-            lame_set_ATHtype( gfp, 3 );
-            lame_set_VBR_min_bitrate_kbps( gfp, 96 );
-		break;
+/*6*/		case LQP_STANDARD:				// --PRESET STANDARD
+		        lame_set_preset( gfp, STANDARD);
+			break;
 
+/*7*/		case LQP_FAST_STANDARD:				// --PRESET FAST STANDARD
+		        lame_set_preset( gfp, STANDARD_FAST);
+			break;
 
-		case LQP_PHONE:
-			lame_set_out_samplerate( gfp, 8000 );
-            lame_set_lowpassfreq( gfp, 3200 );
-			lame_set_lowpasswidth( gfp, 1000 );
-			lame_set_no_short_blocks( gfp, 1 );
-            lame_set_quality( gfp, 5 );
-            lame_set_mode( gfp, MONO );
-			lame_set_brate( gfp, 16 );
-            lame_set_VBR_q( gfp, 6 );
-            lame_set_VBR_min_bitrate_kbps( gfp, 8 );
-            lame_set_VBR_max_bitrate_kbps( gfp, 56 );
-		break;
+/*8*/		case LQP_EXTREME:				// --PRESET EXTREME
+		        lame_set_preset( gfp, EXTREME);
+			break;
+
+/*9*/		case LQP_FAST_EXTREME:				// --PRESET FAST EXTREME:
+		        lame_set_preset( gfp, EXTREME_FAST);
+			break;
+
+/*10*/		case LQP_INSANE:				// --PRESET INSANE
+		        lame_set_preset( gfp, INSANE);
+			break;
+
+/*11*/		case LQP_ABR:					// --PRESET ABR
+				// handled in beInitStream
+			break;
+
+/*12*/		case LQP_CBR:					// --PRESET CBR
+				// handled in beInitStream
+			break;
+
+/*13*/		case LQP_MEDIUM:					// --PRESET MEDIUM
+		        lame_set_preset( gfp, MEDIUM);
+			break;
+
+/*14*/		case LQP_FAST_MEDIUM:					// --PRESET FAST MEDIUM
+		        lame_set_preset( gfp, MEDIUM_FAST);
+			break;
+
+/*1000*/	case LQP_PHONE:
+				lame_set_mode( gfp, MONO );
+		        lame_set_preset( gfp, 16);
+			break;
+
+/*2000*/	case LQP_SW:
+				lame_set_mode( gfp, MONO );
+		        lame_set_preset( gfp, 24);
+			break;
+
+/*3000*/	case LQP_AM:
+				lame_set_mode( gfp, MONO );
+		        lame_set_preset( gfp, 40);
+			break;
+
+/*4000*/	case LQP_FM:
+		        lame_set_preset( gfp, 112);
+			break;
+
+/*5000*/	case LQP_VOICE:
+				lame_set_mode( gfp, MONO );
+		        lame_set_preset( gfp, 56);
+			break;
+
+/*6000*/	case LQP_RADIO:
+		        lame_set_preset( gfp, 112);
+			break;
+
+/*7000*/	case LQP_TAPE:
+		        lame_set_preset( gfp, 112);
+			break;
+
+/*8000*/	case LQP_HIFI:
+		        lame_set_preset( gfp, 160);
+			break;
+
+/*9000*/	case LQP_CD:
+		        lame_set_preset( gfp, 192);
+			break;
+
+/*10000*/	case LQP_STUDIO:
+		        lame_set_preset( gfp, 256);
+			break;
 
-		case LQP_SW:
-			lame_set_out_samplerate( gfp, 11025 );
-            lame_set_lowpassfreq( gfp, 4800 );
-			lame_set_lowpasswidth( gfp, 500 );
-			lame_set_quality( gfp, 5 );
-            lame_set_mode( gfp, MONO );
-			lame_set_brate( gfp, 24 );
-            lame_set_VBR_q( gfp, 5 );
-            lame_set_VBR_min_bitrate_kbps( gfp, 8 );
-            lame_set_VBR_max_bitrate_kbps( gfp, 64 );
-		break;
-		case LQP_AM:
-			lame_set_out_samplerate( gfp, 16000 );
-            lame_set_lowpassfreq( gfp, 7200 );
-			lame_set_lowpasswidth( gfp, 500 );
-			lame_set_quality( gfp, 5 );
-            lame_set_mode( gfp, MONO );
-			lame_set_brate( gfp, 32 );
-            lame_set_VBR_q( gfp, 5 );
-            lame_set_VBR_min_bitrate_kbps( gfp, 16 );
-            lame_set_VBR_max_bitrate_kbps( gfp, 128 );
-		break;
-		case LQP_FM:
-			lame_set_out_samplerate( gfp, 22050 );
-            lame_set_lowpassfreq( gfp, 9950 );
-			lame_set_lowpasswidth( gfp, 880 );
-			lame_set_quality( gfp, 5 );
-            lame_set_mode( gfp, JOINT_STEREO );
-			lame_set_brate( gfp, 64 );
-            lame_set_VBR_q( gfp, 5 );
-            lame_set_VBR_min_bitrate_kbps( gfp, 24 );
-            lame_set_VBR_max_bitrate_kbps( gfp, 160 );
-		break;
-		case LQP_VOICE:
-			lame_set_out_samplerate( gfp, 32000 );
-            lame_set_lowpassfreq( gfp, 12300 );
-			lame_set_lowpasswidth( gfp, 2000 );
-			lame_set_no_short_blocks( gfp, 1 );
-			lame_set_quality( gfp, 5 );
-            lame_set_mode( gfp, MONO );
-			lame_set_brate( gfp, 56 );
-            lame_set_VBR_q( gfp, 5 );
-            lame_set_VBR_min_bitrate_kbps( gfp, 32 );
-            lame_set_VBR_max_bitrate_kbps( gfp, 128 );
-		break;
-		case LQP_RADIO:
-            lame_set_lowpassfreq( gfp, 15000 );
-			lame_set_lowpasswidth( gfp, 0 );
-			lame_set_quality( gfp, 5 );
-            lame_set_mode( gfp, JOINT_STEREO );
-			lame_set_brate( gfp, 112 );
-            lame_set_VBR_q( gfp, 4 );
-            lame_set_VBR_min_bitrate_kbps( gfp, 64 );
-            lame_set_VBR_max_bitrate_kbps( gfp, 256 );
-		break;
-		case LQP_TAPE:
-            lame_set_lowpassfreq( gfp, 18500 );
-			lame_set_lowpasswidth( gfp, 2000 );
-			lame_set_quality( gfp, 5 );
-            lame_set_mode( gfp, JOINT_STEREO );
-			lame_set_brate( gfp, 128 );
-            lame_set_VBR_q( gfp, 4 );
-            lame_set_VBR_min_bitrate_kbps( gfp, 96 );
-            lame_set_VBR_max_bitrate_kbps( gfp, 320 );
-		break;
-		case LQP_HIFI:
-            lame_set_lowpassfreq( gfp, 20240 );
-			lame_set_lowpasswidth( gfp, 2200 );
-			lame_set_quality( gfp, 2 );
-            lame_set_mode( gfp, JOINT_STEREO );
-			lame_set_brate( gfp, 160 );
-            lame_set_VBR_q( gfp, 3 );
-            lame_set_VBR_min_bitrate_kbps( gfp, 112 );
-            lame_set_VBR_max_bitrate_kbps( gfp, 320 );
-		break;
-		case LQP_CD:
-            lame_set_lowpassfreq( gfp, -1 );
-            lame_set_highpassfreq( gfp, -1 );
-			lame_set_quality( gfp, 2 );
-            lame_set_mode( gfp, STEREO );
-			lame_set_brate( gfp, 192 );
-            lame_set_VBR_q( gfp, 2 );
-            lame_set_VBR_min_bitrate_kbps( gfp, 128 );
-            lame_set_VBR_max_bitrate_kbps( gfp, 320 );
-		break;
-		case LQP_STUDIO:
-            lame_set_lowpassfreq( gfp, -1 );
-            lame_set_highpassfreq( gfp, -1 );
-			lame_set_quality( gfp, 2 );
-            lame_set_mode( gfp, STEREO );
-			lame_set_brate( gfp, 256 );
-            lame_set_VBR_q( gfp, 0 );
-            lame_set_VBR_min_bitrate_kbps( gfp, 160 );
-            lame_set_VBR_max_bitrate_kbps( gfp, 320 );
-		break;
 	}
 }
 
 
 __declspec(dllexport) BE_ERR	beInitStream(PBE_CONFIG pbeConfig, PDWORD dwSamples, PDWORD dwBufferSize, PHBE_STREAM phbeStream)
 {
-	int			nDllArgC=0;
-	BE_CONFIG	lameConfig = { 0, };
-	int			nInitReturn = 0;
+	int actual_bitrate;
+//2001-12-18
+	int					nDllArgC = 0;
+	BE_CONFIG			lameConfig = { 0, };
+	int					nInitReturn = 0;
+	lame_global_flags*	gfp = NULL;
 
 	// Init the global flags structure
 	gfp = lame_init();
+	*phbeStream = (HBE_STREAM)gfp;
 
 	// clear out structure
 	memset(&lameConfig,0x00,CURRENT_STRUCT_SIZE);
@@ -261,7 +237,7 @@ __declspec(dllexport) BE_ERR	beInitStream(PBE_CONFIG pbeConfig, PDWORD dwSamples
 	// Check if this is a regular BLADE_ENCODER header
 	if (pbeConfig->dwConfig!=BE_CONFIG_LAME)
 	{
-		int	nCRC=pbeConfig->format.mp3.bCRC;
+		int nCRC=pbeConfig->format.mp3.bCRC;
 		int nVBR=(nCRC>>12)&0x0F;
 
 		// Copy parameter from old Blade structure
@@ -274,7 +250,7 @@ __declspec(dllexport) BE_ERR	beInitStream(PBE_CONFIG pbeConfig, PDWORD dwSamples
 		lameConfig.format.LHV1.dwBitrate	=pbeConfig->format.mp3.wBitrate;
 		lameConfig.format.LHV1.bPrivate		=pbeConfig->format.mp3.bPrivate;
 		lameConfig.format.LHV1.bOriginal	=pbeConfig->format.mp3.bOriginal;
-		lameConfig.format.LHV1.bCRC			=nCRC&0x01;
+		lameConfig.format.LHV1.bCRC		=nCRC&0x01;
 		lameConfig.format.LHV1.bCopyright	=pbeConfig->format.mp3.bCopyright;
 	
 		// Fill out the unknowns
@@ -299,38 +275,73 @@ __declspec(dllexport) BE_ERR	beInitStream(PBE_CONFIG pbeConfig, PDWORD dwSamples
 		memcpy(&lameConfig,pbeConfig,pbeConfig->format.LHV1.dwStructSize);
 	}
 
-
-	// Not used, always assign stream 1
-	*phbeStream=1;
-
-
 	// --------------- Set arguments to LAME encoder -------------------------
 
 	// Set input sample frequency
 	lame_set_in_samplerate( gfp, lameConfig.format.LHV1.dwSampleRate );
 
-	// The following settings only use when preset is not one of the new LAME QUALITY Presets
-	if ( (int)lameConfig.format.LHV1.nPreset < (int) LQP_PHONE )
+        // disable INFO/VBR tag by default.  
+        // if this tag is used, the calling program must call beWriteVBRTag()
+        // after encoding.  But the original DLL documentation does not 
+        // require the 
+        // app to call beWriteVBRTag() unless they have specifically
+        // set LHV1.bWriteVBRHeader=TRUE.  Thus the default setting should
+        // be disabled.  
+	lame_set_bWriteVbrTag( gfp, 0 );
+
+//2001-12-18 Dibrom's ABR preset stuff
+
+	if(lameConfig.format.LHV1.nPreset == LQP_ABR)		// --ALT-PRESET ABR
 	{
-  		switch ( lameConfig.format.LHV1.nMode )
+		actual_bitrate = lameConfig.format.LHV1.dwVbrAbr_bps / 1000;
+
+		// limit range
+		if( actual_bitrate > 320)
+		{
+			actual_bitrate = 320;
+		}
+
+		if( actual_bitrate < 8 )
+		{
+			actual_bitrate = 8;
+		}
+
+		lame_set_preset( gfp, actual_bitrate );
+	}    
+
+// end Dibrom's ABR preset 2001-12-18 ****** START OF CBR
+
+	if(lameConfig.format.LHV1.nPreset == LQP_CBR)		// --ALT-PRESET CBR
+	{
+		actual_bitrate = lameConfig.format.LHV1.dwBitrate;
+        lame_set_preset(gfp, actual_bitrate);
+        lame_set_VBR(gfp, vbr_off);
+	}
+
+// end Dibrom's CBR preset 2001-12-18
+
+	// The following settings only used when preset is not one of the LAME QUALITY Presets
+	if ( (int)lameConfig.format.LHV1.nPreset < (int) LQP_STANDARD )
+	{
+		switch ( lameConfig.format.LHV1.nMode )
 		{
 			case BE_MP3_MODE_STEREO:
-	            lame_set_mode( gfp, STEREO );
+				lame_set_mode( gfp, STEREO );
 				lame_set_num_channels( gfp, 2 );
-			break;
+				break;
 			case BE_MP3_MODE_JSTEREO:
-	            lame_set_mode( gfp, JOINT_STEREO );
+				lame_set_mode( gfp, JOINT_STEREO );
 				lame_set_num_channels( gfp, 2 );
-			break;
+				break;
 			case BE_MP3_MODE_MONO:
-	            lame_set_mode( gfp, MONO );
+				lame_set_mode( gfp, MONO );
 				lame_set_num_channels( gfp, 1 );
-			break;
-			case BE_MP3_MODE_DUALCHANNEL:
+				break;
+			case BE_MP3_MODE_DUALCHANNEL: //warning: there is NO dual channel option working in Lame
 				lame_set_force_ms( gfp, 1 );
-	            lame_set_mode( gfp, JOINT_STEREO );
+				lame_set_mode( gfp, STEREO );
 				lame_set_num_channels( gfp, 2 );
-			break;
+				break;
 			default:
 			{
 				DebugPrintf("Invalid lameConfig.format.LHV1.nMode, value is %d\n",lameConfig.format.LHV1.nMode);
@@ -340,43 +351,63 @@ __declspec(dllexport) BE_ERR	beInitStream(PBE_CONFIG pbeConfig, PDWORD dwSamples
 
 		if ( lameConfig.format.LHV1.bEnableVBR )
 		{
-			lame_set_VBR( gfp, vbr_default );
-
+			/* set VBR quality */
 			lame_set_VBR_q( gfp, lameConfig.format.LHV1.nVBRQuality );
 
-			if (lameConfig.format.LHV1.bWriteVBRHeader==TRUE)
+			/* select proper VBR method */
+			switch ( lameConfig.format.LHV1.nVbrMethod)
 			{
-				lame_set_bWriteVbrTag( gfp, 1 );
-			}
-			else
-			{
-				lame_set_bWriteVbrTag( gfp, 0 );
+				case VBR_METHOD_NONE:
+					lame_set_VBR( gfp, vbr_off );
+				break;
+
+				case VBR_METHOD_DEFAULT:
+					lame_set_VBR( gfp, vbr_default ); 
+				break;
+
+				case VBR_METHOD_OLD:
+					lame_set_VBR( gfp, vbr_rh ); 
+				break;
+
+				case VBR_METHOD_MTRH:
+				case VBR_METHOD_NEW:
+					/*                                
+					 * the --vbr-mtrh commandline switch is obsolete. 
+					 * now --vbr-mtrh is known as --vbr-new
+					 */
+					lame_set_VBR( gfp, vbr_mtrh ); 
+				break;
+
+				case VBR_METHOD_ABR:
+					lame_set_VBR( gfp, vbr_abr ); 
+				break;
+
+				default:
+					/* unsupported VBR method */
+					assert( FALSE );
 			}
 		}
 		else
 		{
+			/* use CBR encoding method, so turn off VBR */
 			lame_set_VBR( gfp, vbr_off );
 		}
 
-		// Set frequency resampling rate, if specified
-		if ( lameConfig.format.LHV1.dwReSampleRate > 0 )
-		{
-			lame_set_out_samplerate( gfp, lameConfig.format.LHV1.dwReSampleRate );
-		}
-		
-		// Set bitrate.  (CDex users always specify bitrate=Min bitrate when using VBR)
+		/* Set bitrate.  (CDex users always specify bitrate=Min bitrate when using VBR) */
 		lame_set_brate( gfp, lameConfig.format.LHV1.dwBitrate );
 			
-		// Use ABR?
-		if (lameConfig.format.LHV1.dwVbrAbr_bps>0)
+		/* check if we have to use ABR, in order to backwards compatible, this
+		 * condition should still be checked indepedent of the nVbrMethod method
+		 */
+		if (lameConfig.format.LHV1.dwVbrAbr_bps > 0 )
 		{
-			// set VBR to ABR
+			/* set VBR method to ABR */
 			lame_set_VBR( gfp, vbr_abr );
 
-			// calculate to kbps
+			/* calculate to kbps, round to nearest kbps */
 			lame_set_VBR_mean_bitrate_kbps( gfp, ( lameConfig.format.LHV1.dwVbrAbr_bps + 500 ) / 1000 );
 
-			// limit range
+			/* limit range */
 			if( lame_get_VBR_mean_bitrate_kbps( gfp ) > 320)
 			{
 				lame_set_VBR_mean_bitrate_kbps( gfp, 320 );
@@ -388,43 +419,36 @@ __declspec(dllexport) BE_ERR	beInitStream(PBE_CONFIG pbeConfig, PDWORD dwSamples
 			}
 		}
 
-
-		if ( lameConfig.format.LHV1.bEnableVBR )
-		{
-			switch ( lameConfig.format.LHV1.nVbrMethod)
-			{
-				case VBR_METHOD_NONE:
-					lame_set_VBR( gfp, vbr_off );
-				break;
-
-				case VBR_METHOD_DEFAULT:
-		            lame_set_VBR( gfp, vbr_default ); 
-				break;
-
-				case VBR_METHOD_OLD:
-		            lame_set_VBR( gfp, vbr_rh ); 
-				break;
-
-				case VBR_METHOD_NEW:
-/*                                
-		            lame_set_VBR( gfp, vbr_mt ); 
-				break;
-the --vbr-mtrh commandline switch is obsolete. 
-now --vbr-mtrh is known as --vbr-new
-*/
-				case VBR_METHOD_MTRH:
-		            lame_set_VBR( gfp, vbr_mtrh ); 
-				break;
-
-				case VBR_METHOD_ABR:
-		            lame_set_VBR( gfp, vbr_abr ); 
-				break;
-				default:
-					assert( FALSE );
-
-			}
-		}
 	}
+
+    // First set all the preset options
+    if ( LQP_NOPRESET !=  lameConfig.format.LHV1.nPreset )
+    {
+		PresetOptions( gfp, lameConfig.format.LHV1.nPreset );
+    }
+
+
+	// Set frequency resampling rate, if specified
+	if ( lameConfig.format.LHV1.dwReSampleRate > 0 )
+	{
+		lame_set_out_samplerate( gfp, lameConfig.format.LHV1.dwReSampleRate );
+	}
+		
+
+    switch ( lameConfig.format.LHV1.nMode )
+	{
+		case BE_MP3_MODE_MONO:
+			lame_set_mode( gfp, MONO );
+			lame_set_num_channels( gfp, 1 );
+		break;
+
+		default:
+            break;
+	}
+
+
+	// Use strict ISO encoding?
+	lame_set_strict_ISO( gfp, ( lameConfig.format.LHV1.bStrictIso == TRUE ) ? 1 : 0 );
 	
 	// Set copyright flag?
 	if ( lameConfig.format.LHV1.bCopyright )
@@ -463,12 +487,6 @@ now --vbr-mtrh is known as --vbr-new
 	}
 	
 
-	// First set all the preset options
-	if ( LQP_NOPRESET !=  lameConfig.format.LHV1.nPreset )
-	{
-		PresetOptions( gfp, lameConfig.format.LHV1.nPreset );
-	}
-
 	// Set VBR min bitrate, if specified
 	if ( lameConfig.format.LHV1.dwBitrate > 0 )
 	{
@@ -480,13 +498,30 @@ now --vbr-mtrh is known as --vbr-new
 	{
 		lame_set_VBR_max_bitrate_kbps( gfp, lameConfig.format.LHV1.dwMaxBitrate );
 	}
-
-	if (lameConfig.format.LHV1.bNoRes)
+	// Set bit resovoir option
+	if ( lameConfig.format.LHV1.bNoRes )
 	{
 		lame_set_disable_reservoir( gfp,1 );
 		lame_set_padding_type( gfp, PAD_NO );
 	}
 
+	// check if the VBR tag is required
+	if ( TRUE == lameConfig.format.LHV1.bWriteVBRHeader) 
+	{
+		lame_set_bWriteVbrTag( gfp, 1 );
+	}
+	else
+	{
+		lame_set_bWriteVbrTag( gfp, 0 );
+	}
+
+	// Override Quality setting, use HIGHBYTE = NOT LOWBYTE to be backwards compatible
+	if (	( lameConfig.format.LHV1.nQuality & 0xFF ) ==
+			~( lameConfig.format.LHV1.nQuality >> 8 ) )
+	{
+		lame_set_quality( gfp, lameConfig.format.LHV1.nQuality & 0xFF );
+	}
+	
 	if ( 0 != ( nInitReturn = lame_init_params( gfp ) ) )
 	{
 		return nInitReturn;
@@ -511,17 +546,42 @@ now --vbr-mtrh is known as --vbr-new
 	*dwBufferSize=(DWORD)( 1.25 * ( *dwSamples / lame_get_num_channels( gfp ) ) + 7200 );
 
 	// For debugging purposes
-	dump_config( );
+	dump_config( gfp );
 
 	// Everything went OK, thus return SUCCESSFUL
 	return BE_ERR_SUCCESSFUL;
 }
 
 
+
+__declspec(dllexport) BE_ERR	beFlushNoGap(HBE_STREAM hbeStream, PBYTE pOutput, PDWORD pdwOutput)
+{
+	int nOutputSamples = 0;
+
+	lame_global_flags*	gfp = (lame_global_flags*)hbeStream;
+
+	// Init the global flags structure
+    nOutputSamples = lame_encode_flush_nogap( gfp, pOutput, LAME_MAXMP3BUFFER );
+
+	if ( nOutputSamples < 0 )
+	{
+		*pdwOutput = 0;
+		return BE_ERR_BUFFER_TOO_SMALL;
+	}
+	else
+	{
+		*pdwOutput = nOutputSamples;
+	}
+
+	return BE_ERR_SUCCESSFUL;
+}
+
 __declspec(dllexport) BE_ERR	beDeinitStream(HBE_STREAM hbeStream, PBYTE pOutput, PDWORD pdwOutput)
 {
 	int nOutputSamples = 0;
 
+	lame_global_flags*	gfp = (lame_global_flags*)hbeStream;
+
     nOutputSamples = lame_encode_flush( gfp, pOutput, 0 );
 
 	if ( nOutputSamples < 0 )
@@ -533,21 +593,28 @@ __declspec(dllexport) BE_ERR	beDeinitStream(HBE_STREAM hbeStream, PBYTE pOutput,
 	{
 		*pdwOutput = nOutputSamples;
 	}
-
-    
-	// lame will be close in VbrWriteTag function
-	if ( !lame_get_bWriteVbrTag( gfp ) )
-	{
-		// clean up of allocated memory
-		lame_close( gfp );
-	}
-
+  
 	return BE_ERR_SUCCESSFUL;
 }
 
 
 __declspec(dllexport) BE_ERR	beCloseStream(HBE_STREAM hbeStream)
 {
+	lame_global_flags*	gfp = (lame_global_flags*)hbeStream;
+
+	// lame will be close in VbrWriteTag function
+	if ( !lame_get_bWriteVbrTag( gfp ) )
+	{
+		// clean up of allocated memory
+		lame_close( gfp );
+
+		gfp_save = NULL;
+	}
+	else
+	{
+		gfp_save = (lame_global_flags*)hbeStream;
+	}
+
 	// DeInit encoder
 	return BE_ERR_SUCCESSFUL;
 }
@@ -618,10 +685,10 @@ __declspec(dllexport) VOID		beVersion(PBE_VERSION pbeVersion)
 __declspec(dllexport) BE_ERR	beEncodeChunk(HBE_STREAM hbeStream, DWORD nSamples, 
 			 PSHORT pSamples, PBYTE pOutput, PDWORD pdwOutput)
 {
-
 	// Encode it
 	int dwSamples;
 	int	nOutputSamples = 0;
+	lame_global_flags*	gfp = (lame_global_flags*)hbeStream;
 
 	dwSamples = nSamples / lame_get_num_channels( gfp );
 
@@ -664,8 +731,9 @@ __declspec(dllexport) BE_ERR	beEncodeChunkFloatS16NI(HBE_STREAM hbeStream, DWORD
 			PFLOAT buffer_l, PFLOAT buffer_r, PBYTE pOutput, PDWORD pdwOutput)
 {
 	int nOutputSamples;
+	lame_global_flags*	gfp = (lame_global_flags*)hbeStream;
 
-	nOutputSamples = lame_encode_buffer_sample_t(gfp,buffer_l,buffer_r,nSamples,pOutput,0);
+	nOutputSamples = lame_encode_buffer_float(gfp,buffer_l,buffer_r,nSamples,pOutput,0);
 
 	if ( nOutputSamples >= 0 )
 	{
@@ -680,44 +748,56 @@ __declspec(dllexport) BE_ERR	beEncodeChunkFloatS16NI(HBE_STREAM hbeStream, DWORD
 	return BE_ERR_SUCCESSFUL;
 }
 
-
-__declspec(dllexport) BE_ERR beWriteVBRHeader(LPCSTR lpszFileName)
+__declspec(dllexport) BE_ERR beWriteInfoTag( HBE_STREAM hbeStream,
+											 LPCSTR lpszFileName )
 {
-	FILE* fpStream	=NULL;
-	BE_ERR beResult	=BE_ERR_SUCCESSFUL;
+	FILE* fpStream	= NULL;
+	BE_ERR beResult	= BE_ERR_SUCCESSFUL;
 
-	// Do we have to write the VBR tag?
-	if (	( lame_get_bWriteVbrTag( gfp ) ) && 
-			( vbr_off != lame_get_VBR( gfp ) ) )
+	lame_global_flags*	gfp = (lame_global_flags*)hbeStream;
+
+	if ( NULL != gfp )
 	{
-
-		// Try to open the file
-		fpStream=fopen( lpszFileName, "rb+" );
-
-		// Check file open result
-		if ( NULL == fpStream )
+		// Do we have to write the VBR tag?
+		if ( lame_get_bWriteVbrTag( gfp ) )
 		{
-			return BE_ERR_INVALID_FORMAT_PARAMETERS;
+
+			// Try to open the file
+			fpStream=fopen( lpszFileName, "rb+" );
+
+			// Check file open result
+			if ( NULL == fpStream )
+			{
+				beResult = BE_ERR_INVALID_FORMAT_PARAMETERS;
+			}
+			else
+			{
+				// Write Xing header again
+				lame_mp3_tags_fid( gfp, fpStream );
+
+				// Close the file stream
+				fclose( fpStream );
+			}
+
+			// clean up of allocated memory
+			lame_close( gfp );
 		}
-
-		// Write Xing header again
-		lame_mp3_tags_fid( gfp, fpStream );
-
-		// Close the file stream
-		fclose( fpStream );
-
 	}
-
-	if ( lame_get_bWriteVbrTag( gfp ) )
+	else
 	{
-		// clean up of allocated memory
-		lame_close( gfp );
+		beResult = BE_ERR_INVALID_FORMAT_PARAMETERS;
 	}
 
 	// return result
 	return beResult;
 }
 
+// for backwards compatiblity
+__declspec(dllexport) BE_ERR beWriteVBRHeader(LPCSTR lpszFileName)
+{
+	return beWriteInfoTag( (HBE_STREAM)gfp_save, lpszFileName );
+}
+
 
 BOOL APIENTRY DllMain(HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
@@ -742,7 +822,7 @@ BOOL APIENTRY DllMain(HANDLE hModule,
 }
 
 
-static void dump_config( )
+static void dump_config( 	lame_global_flags*	gfp )
 {
 	DebugPrintf("\n\nLame_enc configuration options:\n");
 	DebugPrintf("==========================================================\n");
@@ -760,10 +840,10 @@ static void dump_config( )
 		default:           DebugPrintf( "Error (unknown)\n" ); break;
 	}
 
-	DebugPrintf("sampling frequency     =%.1f kHz\n", lame_get_in_samplerate( gfp ) /1000.0 );
+	DebugPrintf("Input sample rate      =%.1f kHz\n", lame_get_in_samplerate( gfp ) /1000.0 );
+	DebugPrintf("Output sample rate     =%.1f kHz\n", lame_get_out_samplerate( gfp ) /1000.0 );
+
 	DebugPrintf("bitrate                =%d kbps\n", lame_get_brate( gfp ) );
-	DebugPrintf("Vbr Min bitrate        =%d kbps\n", lame_get_VBR_min_bitrate_kbps( gfp ) );
-	DebugPrintf("Vbr Max bitrate        =%d kbps\n", lame_get_VBR_max_bitrate_kbps( gfp ) );
 	DebugPrintf("Quality Setting        =%d\n", lame_get_quality( gfp ) );
 
 	DebugPrintf("Low pass frequency     =%d\n", lame_get_lowpassfreq( gfp ) );
@@ -772,7 +852,8 @@ static void dump_config( )
 	DebugPrintf("High pass frequency    =%d\n", lame_get_highpassfreq( gfp ) );
 	DebugPrintf("High pass width        =%d\n", lame_get_highpasswidth( gfp ) );
 
-	DebugPrintf("No Short Blocks        =%d\n", lame_get_no_short_blocks( gfp ) );
+	DebugPrintf("No short blocks        =%d\n", lame_get_no_short_blocks( gfp ) );
+	DebugPrintf("Force short blocks     =%d\n", lame_get_force_short_blocks( gfp ) );
 
 	DebugPrintf("de-emphasis            =%d\n", lame_get_emphasis( gfp ) );
 	DebugPrintf("private flag           =%d\n", lame_get_extension( gfp ) );
@@ -784,6 +865,11 @@ static void dump_config( )
 	DebugPrintf("Force mid/side stereo  =%s\n", ( lame_get_force_ms( gfp ) )?"enabled":"disabled" );
 	DebugPrintf("Padding Type           =%d\n", lame_get_padding_type( gfp ) );
 	DebugPrintf("Disable Resorvoir      =%d\n", lame_get_disable_reservoir( gfp ) );
+	DebugPrintf("Allow diff-short       =%d\n", lame_get_allow_diff_short( gfp ) );
+	DebugPrintf("Temporal masking effect=%d\n", lame_get_interChRatio( gfp ) );
+	DebugPrintf("Strict ISO Encoding    =%s\n", ( lame_get_strict_ISO( gfp ) ) ?"Yes":"No");
+	DebugPrintf("Scale                  =%5.2f\n", lame_get_scale( gfp ) );
+
 	DebugPrintf("VBR                    =%s, VBR_q =%d, VBR method =",
 					( lame_get_VBR( gfp ) !=vbr_off ) ? "enabled": "disabled",
 		            lame_get_VBR_q( gfp ) );
@@ -802,7 +888,27 @@ static void dump_config( )
 		break;
 	}
 
+	DebugPrintf("Vbr Min bitrate        =%d kbps\n", lame_get_VBR_min_bitrate_kbps( gfp ) );
+	DebugPrintf("Vbr Max bitrate        =%d kbps\n", lame_get_VBR_max_bitrate_kbps( gfp ) );
+
 	DebugPrintf("Write VBR Header       =%s\n", ( lame_get_bWriteVbrTag( gfp ) ) ?"Yes":"No");
+	DebugPrintf("VBR Hard min           =%d\n", lame_get_VBR_hard_min( gfp ) );
+
+	DebugPrintf("ATH Only               =%d\n", lame_get_ATHonly( gfp ) );
+	DebugPrintf("ATH short              =%d\n", lame_get_ATHshort( gfp ) );
+	DebugPrintf("ATH short              =%d\n", lame_get_ATHshort( gfp ) );
+	DebugPrintf("ATH no                 =%d\n", lame_get_noATH( gfp ) );
+	DebugPrintf("ATH type               =%d\n", lame_get_ATHtype( gfp ) );
+	DebugPrintf("ATH lower              =%d\n", lame_get_ATHlower( gfp ) );
+	DebugPrintf("ATH aa                 =%d\n", lame_get_athaa_type( gfp ) );
+	DebugPrintf("ATH aa  loudapprox     =%d\n", lame_get_athaa_loudapprox( gfp ) );
+	DebugPrintf("ATH aa  sensitivity    =%d\n", lame_get_athaa_sensitivity( gfp ) );
+	DebugPrintf("ATHtype                =%d\n", lame_get_ATHtype( gfp ) );
+
+	DebugPrintf("Experimental nspsytune =%d\n", lame_get_exp_nspsytune( gfp ) );
+	DebugPrintf("Experimental X         =%d\n", lame_get_experimentalX( gfp ) );
+	DebugPrintf("Experimental Y         =%d\n", lame_get_experimentalY( gfp ) );
+	DebugPrintf("Experimental Z         =%d\n", lame_get_experimentalZ( gfp ) );
 }
 
 
