@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: encoder.c,v 1.66 2001/12/20 08:44:04 dibrom Exp $ */
+/* $Id: encoder.c,v 1.77 2002/09/30 16:49:30 bouvigne Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -75,29 +75,28 @@ adjust_ATH( lame_global_flags* const  gfp,
         break;
     
     case 2:                     /* jd - 2001 mar 12, 27, jun 30 */
-    {				/* loudness based on equal loudness curve; */
+    {				            /* loudness based on equal loudness curve; */
                                 /* use granule with maximum combined loudness*/
-	FLOAT gr2_max;
+	    FLOAT gr2_max;
         max_pow = gfc->loudness_sq[0][0];
+	    gr2_max = gfc->loudness_sq[1][0];
         if( gfc->channels_out == 2 ) {
             max_pow += gfc->loudness_sq[0][1];
-	    gr2_max = gfc->loudness_sq[1][0] + gfc->loudness_sq[1][1];
-	} else {
-	    gr2_max = gfc->loudness_sq[1][0];
-	    max_pow += max_pow;
-	    gr2_max += gr2_max;
-	}
-	if( gfc->mode_gr == 2 ) {
-	    max_pow = Max( max_pow, gr2_max );
-	}
-	max_pow *= 0.5;		/* max_pow approaches 1.0 for full band noise*/
+	        gr2_max += gfc->loudness_sq[1][1];
+	    } else {
+	        max_pow += max_pow;
+	        gr2_max += gr2_max;
+	    }
+	    if( gfc->mode_gr == 2 ) {
+	        max_pow = Max( max_pow, gr2_max );
+	    }
+	    max_pow *= 0.5;		    /* max_pow approaches 1.0 for full band noise*/
         break;
     }
 
-    default:                    /* jd - 2001 mar 27, 31, jun 30 */
-                                /* no adaptive threshold */
-        max_pow = 1.0 / gfc->athaa_sensitivity_p;
-        break;
+    default:
+        gfc->ATH->adjust = 1.0;	/* no adjustment */
+        return;
     }
 
                                 /* jd - 2001 mar 31, jun 30 */
@@ -108,18 +107,18 @@ adjust_ATH( lame_global_flags* const  gfp,
         max_pow_alt *= pow( 10.0, gfc->presetTune.athadjust_safe_athaasensitivity / -10.0 );
 
     /*  adjust ATH depending on range of maximum value
+        The first mode was just implemented for rough
+        experimentation. Third mode is recommended
      */
     switch ( gfc->ATH->use_adjust ) {
 
-    case  1:
-        max_val = sqrt( max_pow ); /* GB's original code requires a maximum */
-        max_val *= 32768;          /*  sample or loudness value up to 32768 */
-
-                                /* by Gabriel Bouvigne */
-        if      (0.5 < max_val / 32768) {       /* value above 50 % */
+    case  1:                                    /* by Gabriel Bouvigne */
+        max_val = sqrt( max_pow );              /* max_val should be between 0 and 1 */
+                                                
+        if      (max_val > 0.5) {               /* value above 50 % */
                 gfc->ATH->adjust = 1.0;         /* do not reduce ATH */
         }
-        else if (0.3 < max_val / 32768) {       /* value above 30 % */
+        else if (max_val > 0.3) {               /* value above 30 % */
                 gfc->ATH->adjust *= 0.955;      /* reduce by ~0.2 dB */
                 if (gfc->ATH->adjust < 0.3)     /* but ~5 dB in maximum */
                     gfc->ATH->adjust = 0.3;            
@@ -161,26 +160,20 @@ adjust_ATH( lame_global_flags* const  gfp,
         if( max_pow > 0.03125) { /* ((1 - 0.000625)/ 31.98) from curve below */
             if( gfc->ATH->adjust >= 1.0) {
                 gfc->ATH->adjust = 1.0;
-                if (gfc->presetTune.use) {
-		        if (max_pow_alt > gfc->presetTune.athadjust_safe_noiseshaping_thre)
-			      gfc->presetTune.athadjust_safe_noiseshaping = 1;
-		        else
-			      gfc->presetTune.athadjust_safe_noiseshaping = 0;
-                }
             } else {
                                 /* preceding frame has lower ATH adjust; */
                                 /* ascend only to the preceding adjust_limit */
                                 /* in case there is leading low volume */
                 if( gfc->ATH->adjust < gfc->ATH->adjust_limit) {
                     gfc->ATH->adjust = gfc->ATH->adjust_limit;
-                    if (gfc->presetTune.use) {
-                        if (max_pow_alt > gfc->presetTune.athadjust_safe_noiseshaping_thre)
-                            gfc->presetTune.athadjust_safe_noiseshaping = 1;
-                        else
-                            gfc->presetTune.athadjust_safe_noiseshaping = 0;
-                    }
                 }
             }
+            if (gfc->presetTune.use) {
+		        if (max_pow_alt > gfc->presetTune.athadjust_safe_noiseshaping_thre)
+			      gfc->presetTune.athadjust_safe_noiseshaping = 1;
+		        else
+			      gfc->presetTune.athadjust_safe_noiseshaping = 0;
+            }
             gfc->ATH->adjust_limit = 1.0;
         } else {                /* adjustment curve */
                                 /* about 32 dB maximum adjust (0.000625) */
@@ -255,22 +248,14 @@ int  lame_encode_mp3_frame (				// Output
 	unsigned char*            mp3buf, 		// Output
 	int                    mp3buf_size )		// Output
 {
-#ifdef macintosh /* PLL 14/04/2000 */
-  static FLOAT8 xr[2][2][576];
-  static int l3_enc[2][2][576];
-#else
-  FLOAT8 xr[2][2][576];
-  int l3_enc[2][2][576];
-#endif
   int mp3count;
   III_psy_ratio masking_LR[2][2];    /*LR masking & energy */
   III_psy_ratio masking_MS[2][2]; /*MS masking & energy */
   III_psy_ratio (*masking)[2][2];  /*pointer to selected maskings*/
-  III_scalefac_t scalefac[2][2];
   const sample_t *inbuf[2];
   lame_internal_flags *gfc=gfp->internal_flags;
 
-  FLOAT8 tot_ener[2][4];   
+  FLOAT8 tot_ener[2][4];
   FLOAT8 ms_ener_ratio[2]={.5,.5};
   chgrdata pe,pe_MS;
   chgrdata *pe_use;
@@ -285,7 +270,6 @@ int  lame_encode_mp3_frame (				// Output
 
   memset((char *) masking_LR, 0, sizeof(masking_LR));
   memset((char *) masking_MS, 0, sizeof(masking_MS));
-  memset((char *) scalefac, 0, sizeof(scalefac));
   inbuf[0]=inbuf_l;
   inbuf[1]=inbuf_r;
 
@@ -307,6 +291,16 @@ int  lame_encode_mp3_frame (				// Output
     gfc->frac_SpF = ((gfp->version+1)*72000L*gfp->brate) % gfp->out_samplerate;
     gfc->slot_lag  = gfc->frac_SpF;
     
+    switch (gfp->padding_type) {
+    default:
+    case PAD_NO:
+        gfc->padding = FALSE;
+        break;
+    case PAD_ALL:
+        gfc->padding = TRUE;
+        break;
+    }
+    
     /* check FFT will not use a negative starting offset */
 #if 576 < FFTOFFSET
 # error FFTOFFSET greater than 576: FFT uses a negative offset
@@ -338,10 +332,10 @@ int  lame_encode_mp3_frame (				// Output
       /* polyphase filtering / mdct */
       for ( gr = 0; gr < gfc->mode_gr; gr++ ) {
 	for ( ch = 0; ch < gfc->channels_out; ch++ ) {
-	  gfc->l3_side.gr[gr].ch[ch].tt.block_type=SHORT_TYPE;
+	  gfc->l3_side.tt[gr][ch].block_type=SHORT_TYPE;
 	}
       }
-      mdct_sub48(gfc, primebuff0, primebuff1, xr);
+      mdct_sub48(gfc, primebuff0, primebuff1);
     }
     
     iteration_init(gfp);
@@ -358,23 +352,7 @@ int  lame_encode_mp3_frame (				// Output
 
 
   /********************** padding *****************************/
-  switch (gfp->padding_type) {
-  case PAD_NO:
-    gfc->padding = FALSE;
-    break;
-  case PAD_ALL:
-    gfc->padding = TRUE;
-    break;
-  case PAD_ADJUST:
-  default:
-    if (gfp->VBR!=vbr_off) {
-      gfc->padding = FALSE;
-    } else {
-      if (gfp->disable_reservoir) {
-	gfc->padding = FALSE;
-	/* if the user specified --nores, dont very gfc->padding either */
-	/* tiny changes in frac_SpF rounding will cause file differences */
-      }else{
+  if (gfp->padding_type == PAD_ADJUST) {
         /* padding method as described in 
          * "MPEG-Layer3 / Bitstream Syntax and Decoding"
          * by Martin Sieler, Ralph Sperschneider
@@ -386,13 +364,12 @@ int  lame_encode_mp3_frame (				// Output
 
         gfc->slot_lag -= gfc->frac_SpF;
         if (gfc->slot_lag < 0) {
-          gfc->slot_lag += gfp->out_samplerate;
-          gfc->padding = TRUE;
-        } else {
-          gfc->padding = FALSE;
+            gfc->slot_lag += gfp->out_samplerate;
+            gfc->padding = TRUE;
+        } 
+        else {
+            gfc->padding = FALSE;
         }
-      } /* reservoir enabled */
-    }
   }
 
 
@@ -425,7 +402,7 @@ int  lame_encode_mp3_frame (				// Output
       if (ret!=0) return -4;
 
       for ( ch = 0; ch < gfc->channels_out; ch++ )
-	gfc->l3_side.gr[gr].ch[ch].tt.block_type=blocktype[ch];
+	gfc->l3_side.tt[gr][ch].block_type=blocktype[ch];
 
       if (check_ms_stereo) {
 	  ms_ener_ratio[gr] = tot_ener[gr][2]+tot_ener[gr][3];
@@ -437,7 +414,7 @@ int  lame_encode_mp3_frame (				// Output
   }else{
     for (gr=0; gr < gfc->mode_gr ; gr++)
       for ( ch = 0; ch < gfc->channels_out; ch++ ) {
-	gfc->l3_side.gr[gr].ch[ch].tt.block_type=NORM_TYPE;
+	gfc->l3_side.tt[gr][ch].block_type=NORM_TYPE;
 	pe_MS[gr][ch]=pe[gr][ch]=700;
       }
   }
@@ -452,36 +429,22 @@ int  lame_encode_mp3_frame (				// Output
   /* block type flags */
   for( gr = 0; gr < gfc->mode_gr; gr++ ) {
     for ( ch = 0; ch < gfc->channels_out; ch++ ) {
-      gr_info *cod_info = &gfc->l3_side.gr[gr].ch[ch].tt;
+      gr_info *cod_info = &gfc->l3_side.tt[gr][ch];
       cod_info->mixed_block_flag = 0;     /* never used by this model */
-      if (cod_info->block_type == NORM_TYPE )
-	cod_info->window_switching_flag = 0;
-      else
-	cod_info->window_switching_flag = 1;
     }
   }
 
 
   /* polyphase filtering / mdct */
-  mdct_sub48(gfc, inbuf[0], inbuf[1], xr);
-  /* re-order the short blocks, for more efficient encoding below */
-  for (gr = 0; gr < gfc->mode_gr; gr++) {
-    for (ch = 0; ch < gfc->channels_out; ch++) {
-      gr_info *cod_info = &gfc->l3_side.gr[gr].ch[ch].tt;
-      if (cod_info->block_type==SHORT_TYPE) {
-	freorder(gfc->scalefac_band.s,xr[gr][ch]);
-      }
-    }
-  }
-  
+  mdct_sub48(gfc, inbuf[0], inbuf[1]);
 
   /* use m/s gfc->channels_out? */
   if (check_ms_stereo) {
     int gr0 = 0, gr1 = gfc->mode_gr-1;
     /* make sure block type is the same in each channel */
     check_ms_stereo =
-      (gfc->l3_side.gr[gr0].ch[0].tt.block_type==gfc->l3_side.gr[gr0].ch[1].tt.block_type) &&
-      (gfc->l3_side.gr[gr1].ch[0].tt.block_type==gfc->l3_side.gr[gr1].ch[1].tt.block_type);
+      (gfc->l3_side.tt[gr0][0].block_type==gfc->l3_side.tt[gr0][1].block_type) &&
+      (gfc->l3_side.tt[gr1][0].block_type==gfc->l3_side.tt[gr1][1].block_type);
   }
   
   /* Here will be selected MS or LR coding of the 2 stereo channels */
@@ -553,9 +516,9 @@ int  lame_encode_mp3_frame (				// Output
       for ( ch = 0; ch < gfc->channels_out; ch++ ) {
 	gfc->pinfo->ms_ratio[gr]=gfc->ms_ratio[gr];
 	gfc->pinfo->ms_ener_ratio[gr]=ms_ener_ratio[gr];
-	gfc->pinfo->blocktype[gr][ch]=
-	  gfc->l3_side.gr[gr].ch[ch].tt.block_type;
-	memcpy(gfc->pinfo->xr[gr][ch],xr[gr][ch],sizeof(xr[gr][ch]));
+	gfc->pinfo->blocktype[gr][ch]=gfc->l3_side.tt[gr][ch].block_type;
+	memcpy(gfc->pinfo->xr[gr][ch], &gfc->l3_side.tt[gr][ch].xr,
+	       sizeof(FLOAT8)*576);
 	/* in psymodel, LR and MS data was stored in pinfo.  
 	   switch to MS data: */
 	if (gfc->mode_ext==MPG_MD_MS_LR) {
@@ -618,24 +581,22 @@ int  lame_encode_mp3_frame (				// Output
   switch (gfp->VBR){ 
   default:
   case vbr_off:
-    iteration_loop( gfp,*pe_use,ms_ener_ratio, xr, *masking, l3_enc, scalefac);
+    iteration_loop( gfp,*pe_use,ms_ener_ratio, *masking);
     break;
   case vbr_mt:
-    VBR_quantize( gfp,*pe_use,ms_ener_ratio, xr, *masking, l3_enc, scalefac);
-    break;
   case vbr_rh:
   case vbr_mtrh:
-    VBR_iteration_loop( gfp,*pe_use,ms_ener_ratio, xr, *masking, l3_enc, scalefac);
+    VBR_iteration_loop( gfp,*pe_use,ms_ener_ratio, *masking);
     break;
   case vbr_abr:
-    ABR_iteration_loop( gfp,*pe_use,ms_ener_ratio, xr, *masking, l3_enc, scalefac);
+    ABR_iteration_loop( gfp,*pe_use,ms_ener_ratio, *masking);
     break;
   }
 
   /*  write the frame to the bitstream  */
   getframebits(gfp, &bitsPerFrame, &mean_bits);
 
-  format_bitstream( gfp, bitsPerFrame, l3_enc, scalefac);
+  format_bitstream( gfp, bitsPerFrame);
 
   /* copy mp3 bit buffer into array */
   mp3count = copy_buffer(gfc,mp3buf,mp3buf_size,1);
@@ -657,7 +618,7 @@ int  lame_encode_mp3_frame (				// Output
 	gfc->pinfo->pcmdata[ch][j] = inbuf[ch][j-FFTOFFSET];
       }
     }
-    set_frame_pinfo (gfp, xr, *masking, l3_enc, scalefac);
+    set_frame_pinfo (gfp, *masking);
   }
 #endif
   
