@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: lame.c,v 1.169 2001/12/14 07:00:57 dibrom Exp $ */
+/* $Id: lame.c,v 1.194 2002/11/28 18:04:18 bouvigne Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -48,6 +48,13 @@
 #include <dmalloc.h>
 #endif
 
+#ifdef __sun__
+/* woraround for SunOS 4.x, it has SEEK_* defined here */
+#include <unistd.h>
+#endif
+
+
+#define DEFAULT_QUALITY 2
 
 static void
 lame_init_params_ppflt_lowpass(FLOAT8 amp_lowpass[32], FLOAT lowpass1,
@@ -455,6 +462,7 @@ lame_init_qval(lame_global_flags * gfp)
         gfc->psymodel = 1;
         gfc->quantization = 1;
         gfc->noise_shaping = 1;
+        gfc->substep_shaping = 1;
         gfc->noise_shaping_amp = 2;
         gfc->noise_shaping_stop = 1;
         gfc->use_best_huffman = 1;
@@ -465,7 +473,8 @@ lame_init_qval(lame_global_flags * gfp)
         gfc->psymodel = 1;
         gfc->quantization = 1;
         gfc->noise_shaping = 1; /* 2=usually lowers quality */
-        gfc->noise_shaping_amp = 3;
+        gfc->substep_shaping = 2;
+        gfc->noise_shaping_amp = 2;
         gfc->noise_shaping_stop = 1;
         gfc->use_best_huffman = 1; /* 2 not yet coded */
     }
@@ -552,10 +561,25 @@ lame_init_params(lame_global_flags * const gfp)
     gfc->report.errorf = gfp->report.errorf;
 
     gfc->CPU_features.i387 = has_i387();
-    gfc->CPU_features.AMD_3DNow = has_3DNow();
-    gfc->CPU_features.MMX = has_MMX();
-    gfc->CPU_features.SIMD = has_SIMD();
-    gfc->CPU_features.SIMD2 = has_SIMD2();
+
+
+    if (gfp->asm_optimizations.amd3dnow )
+        gfc->CPU_features.AMD_3DNow = has_3DNow();
+    else
+        gfc->CPU_features.AMD_3DNow = 0;
+
+    if (gfp->asm_optimizations.mmx )
+        gfc->CPU_features.MMX = has_MMX();
+    else 
+        gfc->CPU_features.MMX = 0;
+
+    if (gfp->asm_optimizations.sse ) {
+        gfc->CPU_features.SIMD = has_SIMD();
+        gfc->CPU_features.SIMD2 = has_SIMD2();
+    } else {
+        gfc->CPU_features.SIMD = 0;
+        gfc->CPU_features.SIMD2 = 0;
+    }
 
 
     if (NULL == gfc->ATH)
@@ -590,6 +614,7 @@ lame_init_params(lame_global_flags * const gfp)
 
     if (gfp->VBR != vbr_off) {
         gfp->free_format = 0; /* VBR can't be mixed with free format */
+        gfp->padding_type = PAD_NO;    
     }
 
     if (gfp->VBR == vbr_off && gfp->brate == 0) {
@@ -981,6 +1006,11 @@ lame_init_params(lame_global_flags * const gfp)
         gfc->nsPsy.sfb21 = pow(10, (i+j) / 4.0 / 10.0);
     }
 
+    if (gfp->exp_nspsytune2.pointer[0]) {
+      gfc->nsPsy.pass1fp = gfp->exp_nspsytune2.pointer[0];
+      gfc->nsPsy.use2 = 1;
+    }
+
     assert( gfp->VBR_q <= 9 );
     assert( gfp->VBR_q >= 0 );
     
@@ -994,7 +1024,7 @@ lame_init_params(lame_global_flags * const gfp)
     case vbr_mtrh:
 
         if (gfp->ATHtype < 0) gfp->ATHtype = 4;
-        if (gfp->quality < 0) gfp->quality = 2;
+        if (gfp->quality < 0) gfp->quality = DEFAULT_QUALITY;
         if (gfp->quality > 7) {
             gfp->quality = 7;     // needs psymodel
             ERRORF( gfc, "VBR needs a psymodel, switching to quality level 7\n");
@@ -1005,43 +1035,64 @@ lame_init_params(lame_global_flags * const gfp)
         if (gfp->cwlimit <= 0) gfp->cwlimit = 0.42 * gfp->out_samplerate;
         gfc->PSY->tonalityPatch = 1;
 
-        if ( gfp->experimentalX <= 4 && gfp->experimentalX >= 0 )
-        {   /* map experimentalX settings to internal secltions */
-            static char const map[] = {2,1,0,3,6};
-            gfc->VBR->quality = map[gfp->experimentalX];
+        switch ( gfp->experimentalX ) {
+        default:
+        case 0: {
+                static const float dbQ[10]={-2.,-1.0,-.66,-.33,0.,0.33,.66,1.0,1.33,1.66};
+                gfc->VBR->quality = 0;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 1;
+            } 
+            break;        
+        case 1: {
+                static float const dbQ[10] = { -2., -1.4, -.7, 0, .7, 1.5, 2.3, 3.1, 4., 5 };
+                gfc->VBR->quality = 1;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 0;    
+            } 
+            break;        
+        case 2: {
+                static float const dbQ[10] = { -1., -.6, -.3, 0, 1, 2, 3, 4, 5, 6};
+                gfc->VBR->quality = 2;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 0;    
+                gfc->PSY->tonalityPatch = 0;
+            } 
+            break;        
+        case 3: {
+                static const float dbQ[10]={-2.,-1.0,-.66,-.33,0.,0.33,.66,1.0,1.33,1.66};
+                gfc->VBR->quality = 3;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 1;
+            } 
+            break;        
+        case 4: {
+                static float const dbQ[10] = { -6,-4.75,-3.5,-2.25,-1,.25,1.5,2.75,4,5.25 };
+                gfc->VBR->quality = 4;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 1;   // not finally
+            }
+            break;        
+        case 5: {
+                static const float dbQ[10]={-2.,-1.0,-.66,-.33,0.,0.33,.66,1.0,1.33,1.66};
+                gfc->VBR->quality = 0;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 2;
+            } 
+            break;        
+        case 9: {
+                static float const dbQ[10] = { -6,-4.75,-3.5,-2.25,-1,.25,1.5,2.75,4,5.25 };
+                gfc->VBR->quality = 4;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 0;   // not finally
+            }
+            break;        
         }
-        else    /* defaulting to */
-        {
-            gfc->VBR->quality = 2;
-        }   
-        if ( gfc->VBR->quality > 5 ) {
-            static float const dbQ[10] = { -6,-4.75,-3.5,-2.25,-1,.25,1.5,2.75,4,5.25 };
-            gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
-            gfc->VBR->smooth = 1;   // not finally
-        }
-        else {
-            static const float dbQ[10]={-2.,-1.0,-.66,-.33,0.,0.33,.66,1.0,1.33,1.66};
-            gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
-            gfc->VBR->smooth = 1;
-        }
-        if ( gfc->VBR->quality == 1 ) {
-            static float const dbQ[10] = { -2., -1.4, -.7, 0, .7, 1.5, 2.3, 3.1, 4., 5 };
-            gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
-            gfc->VBR->smooth = 0;    
-        }
-        if ( gfc->VBR->quality == 0 ) {
-            static float const dbQ[10] = { -1., -.6, -.3, 0, 1, 2, 3, 4, 5, 6};
-            gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
-            gfc->VBR->smooth = 0;    
-            gfc->PSY->tonalityPatch = 0;
-        }
-        
-        gfc->VBR->bitpressure = 1;
         
         if (gfp->experimentalY)
             gfc->sfb21_extra = 0;
         else
-            gfc->sfb21_extra = (gfp->out_samplerate > 44000);
+            gfc->sfb21_extra = (gfp->out_samplerate > 36000);
         
         if ( gfp->athaa_type < 0 )
             gfc->ATH->use_adjust = 3;
@@ -1064,7 +1115,6 @@ lame_init_params(lame_global_flags * const gfp)
                 gfc->VBR->mask_adjust = dbQ[gfp->VBR_q]; 
             }
         }
-        gfc->VBR->bitpressure = 1;
         
         /*  use Gabriel's adaptative ATH shape for VBR by default
          */
@@ -1099,7 +1149,7 @@ lame_init_params(lame_global_flags * const gfp)
         /*  default quality setting is 2
          */
         if (gfp->quality < 0)
-            gfp->quality = 2;
+            gfp->quality = DEFAULT_QUALITY;
 
         break;
 
@@ -1121,10 +1171,10 @@ lame_init_params(lame_global_flags * const gfp)
          */
         gfc->sfb21_extra = 0;
 
-        /*  default quality setting for CBR/ABR is 5
+        /*  default quality setting for CBR/ABR is 2
          */
         if (gfp->quality < 0)
-            gfp->quality = 5;
+            gfp->quality = DEFAULT_QUALITY;
 
         break;
     }
@@ -1160,11 +1210,6 @@ lame_init_params(lame_global_flags * const gfp)
     
     if (gfp->useTemporal < 0 ) gfp->useTemporal = 1;  // on by default
 
-
-    if ( gfp->experimentalY )
-        MSGF(gfc,"\n *** WARNING *** the meaning of the experimental -Y has changed!\n"
-                   "                 now it tells LAME to ignore sfb21 noise shaping (VBR)\n\n");
-
     if ( gfp->preset_expopts && gfc->presetTune.use < 1 )
         MSGF(gfc,"\n*** WARNING ***\n\n"
 		         "Specialized tunings for the preset you are using have been deactivated.\n"
@@ -1209,7 +1254,11 @@ lame_print_config(const lame_global_flags * gfp)
             MSGF(gfc, ", MMX");
 #endif
         if (gfc->CPU_features.AMD_3DNow)
+#ifdef HAVE_NASM
+            MSGF(gfc, ", 3DNow! (ASM used)");
+#else
             MSGF(gfc, ", 3DNow!");
+#endif
         if (gfc->CPU_features.SIMD)
             MSGF(gfc, ", SIMD");
         if (gfc->CPU_features.SIMD2)
@@ -1371,6 +1420,7 @@ lame_print_internals( const lame_global_flags * gfp )
     }
     pc = gfp->useTemporal ? "yes" : "no";
     MSGF( gfc, "\tusing temporal masking effect: %s\n", pc );
+    MSGF( gfc, "\tinterchannel masking ratio: %f\n", gfp->interChRatio );
     MSGF( gfc, "\t...\n" );
     
     /*  that's all ?
@@ -1427,6 +1477,13 @@ lame_encode_frame(lame_global_flags * gfp,
  * mp3buffer_size in bytes = 1.25*num_samples + 7200
  *
  * return code = number of bytes output in mp3buffer.  can be 0
+ *
+ * NOTE: this routine uses LAME's internal PCM data representation,
+ * 'sample_t'.  It should not be used by any application.  
+ * applications should use lame_encode_buffer(), 
+ *                         lame_encode_buffer_float()
+ *                         lame_encode_buffer_int()
+ * etc... depending on what type of data they are working with.  
 */
 int
 lame_encode_buffer_sample_t(lame_global_flags * gfp,
@@ -1457,6 +1514,42 @@ lame_encode_buffer_sample_t(lame_global_flags * gfp,
     in_buffer[1]=buffer_r;  
 
 
+    /* Apply user defined re-scaling */
+
+    /* user selected scaling of the samples */
+    if (gfp->scale != 0 && gfp->scale != 1.0) {
+	for (i=0 ; i<nsamples; ++i) {
+	    in_buffer[0][i] *= gfp->scale;
+	    if (gfc->channels_out == 2)
+		in_buffer[1][i] *= gfp->scale;
+	    }
+    }
+
+    /* user selected scaling of the channel 0 (left) samples */
+    if (gfp->scale_left != 0 && gfp->scale_left != 1.0) {
+	for (i=0 ; i<nsamples; ++i) {
+	    in_buffer[0][i] *= gfp->scale_left;
+	    }
+    }
+
+    /* user selected scaling of the channel 1 (right) samples */
+	if (gfp->scale_right != 0 && gfp->scale_right != 1.0) {
+	    for (i=0 ; i<nsamples; ++i) {
+		in_buffer[1][i] *= gfp->scale_right;
+	    }
+	}
+
+    /* Downsample to Mono if 2 channels in and 1 channel out */
+	if (gfp->num_channels == 2 && gfc->channels_out == 1) {
+		for (i=0; i<nsamples; ++i) {
+			in_buffer[0][i] =
+				0.5 * ((FLOAT8) in_buffer[0][i] + in_buffer[1][i]);
+			in_buffer[1][i] = 0.0;
+		}
+	}
+
+
+
     /* some sanity checks */
 #if ENCDELAY < MDCTDELAY
 # error ENCDELAY is less than MDCTDELAY, see encoder.h
@@ -1472,22 +1565,12 @@ lame_encode_buffer_sample_t(lame_global_flags * gfp,
     mfbuf[0] = gfc->mfbuf[0];
     mfbuf[1] = gfc->mfbuf[1];
 
-    if (gfp->num_channels == 2 && gfc->channels_out == 1) {
-        /* downsample to mono */
-        for (i = 0; i < nsamples; ++i) {
-            in_buffer[0][i] =
-                0.5 * ((FLOAT8) in_buffer[0][i] + in_buffer[1][i]);
-            in_buffer[1][i] = 0.0;
-        }
-    }
-
-
     while (nsamples > 0) {
         int     n_in = 0;    /* number of input samples processed with fill_buffer */
         int     n_out = 0;   /* number of samples output with fill_buffer */
         /* n_in <> n_out if we are resampling */
 
-        /* copy in new samples into mfbuf, with resampling & scaling if necessary */
+        /* copy in new samples into mfbuf, with resampling */
         fill_buffer(gfp, mfbuf, in_buffer, nsamples, &n_in, &n_out);
 
         /* update in_buffer counters */
@@ -1565,7 +1648,7 @@ lame_encode_buffer(lame_global_flags * gfp,
     /* make a copy of input buffer, changing type to sample_t */
     for (i = 0; i < nsamples; i++) {
         in_buffer[0][i] = buffer_l[i];
-        in_buffer[1][i] = buffer_r[i];
+        if (gfc->channels_in>1) in_buffer[1][i] = buffer_r[i];
     }
 
     ret = lame_encode_buffer_sample_t(gfp,in_buffer[0],in_buffer[1],
@@ -1604,7 +1687,7 @@ lame_encode_buffer_float(lame_global_flags * gfp,
     /* make a copy of input buffer, changing type to sample_t */
     for (i = 0; i < nsamples; i++) {
         in_buffer[0][i] = buffer_l[i];
-        in_buffer[1][i] = buffer_r[i];
+        if (gfc->channels_in>1) in_buffer[1][i] = buffer_r[i];
     }
 
     ret = lame_encode_buffer_sample_t(gfp,in_buffer[0],in_buffer[1],
@@ -1644,8 +1727,9 @@ lame_encode_buffer_int(lame_global_flags * gfp,
     /* make a copy of input buffer, changing type to sample_t */
     for (i = 0; i < nsamples; i++) {
                                 /* internal code expects +/- 32768.0 */
-      in_buffer[0][i] = buffer_l[i] * (1.0 / ( 1 << (8 * sizeof(int) - 16)));
-      in_buffer[1][i] = buffer_r[i] * (1.0 / ( 1 << (8 * sizeof(int) - 16)));
+      in_buffer[0][i] = buffer_l[i] * (1.0 / ( 1L << (8 * sizeof(int) - 16)));
+      if (gfc->channels_in>1) 
+         in_buffer[1][i] = buffer_r[i] * (1.0 / ( 1L << (8 * sizeof(int) - 16)));
     }
 
     ret = lame_encode_buffer_sample_t(gfp,in_buffer[0],in_buffer[1],
@@ -1688,7 +1772,8 @@ lame_encode_buffer_long2(lame_global_flags * gfp,
     for (i = 0; i < nsamples; i++) {
                                 /* internal code expects +/- 32768.0 */
       in_buffer[0][i] = buffer_l[i] * (1.0 / ( 1 << (8 * sizeof(long) - 16)));
-      in_buffer[1][i] = buffer_r[i] * (1.0 / ( 1 << (8 * sizeof(long) - 16)));
+      if (gfc->channels_in>1) 
+         in_buffer[1][i] = buffer_r[i] * (1.0 / ( 1 << (8 * sizeof(long) - 16)));
     }
 
     ret = lame_encode_buffer_sample_t(gfp,in_buffer[0],in_buffer[1],
@@ -1729,7 +1814,8 @@ lame_encode_buffer_long(lame_global_flags * gfp,
     /* make a copy of input buffer, changing type to sample_t */
     for (i = 0; i < nsamples; i++) {
         in_buffer[0][i] = buffer_l[i];
-        in_buffer[1][i] = buffer_r[i];
+        if (gfc->channels_in>1) 
+           in_buffer[1][i] = buffer_r[i];
     }
 
     ret = lame_encode_buffer_sample_t(gfp,in_buffer[0],in_buffer[1],
@@ -1757,11 +1843,11 @@ lame_encode_buffer_interleaved(lame_global_flags * gfp,
                                unsigned char *mp3buf, int mp3buf_size)
 {
     int     ret, i;
-    short int *buffer_l;
-    short int *buffer_r;
+    sample_t *buffer_l;
+    sample_t *buffer_r;
 
-    buffer_l = malloc(sizeof(short int) * nsamples);
-    buffer_r = malloc(sizeof(short int) * nsamples);
+    buffer_l = calloc(sizeof(sample_t), nsamples);
+    buffer_r = calloc(sizeof(sample_t), nsamples);
     if (buffer_l == NULL || buffer_r == NULL) {
         return -2;
     }
@@ -1770,7 +1856,7 @@ lame_encode_buffer_interleaved(lame_global_flags * gfp,
         buffer_r[i] = buffer[2 * i + 1];
     }
     ret =
-        lame_encode_buffer(gfp, buffer_l, buffer_r, nsamples, mp3buf,
+        lame_encode_buffer_sample_t(gfp, buffer_l, buffer_r, nsamples, mp3buf,
                            mp3buf_size);
     free(buffer_l);
     free(buffer_r);
@@ -1836,13 +1922,20 @@ lame_init_bitstream(lame_global_flags * gfp)
 	id3tag_write_v2(gfp);
 
     /* initialize histogram data optionally used by frontend */
-    for ( i = 0; i < 16; i++ ) 
+    for ( i = 0; i < 16; i++ ) {
 	gfc->bitrate_stereoMode_Hist [i] [0] =
 	    gfc->bitrate_stereoMode_Hist [i] [1] =
 	    gfc->bitrate_stereoMode_Hist [i] [2] =
 	    gfc->bitrate_stereoMode_Hist [i] [3] =
 	    gfc->bitrate_stereoMode_Hist [i] [4] = 0;
-
+        gfc->bitrate_blockType_Hist [i] [0] =
+            gfc->bitrate_blockType_Hist [i] [1] =
+            gfc->bitrate_blockType_Hist [i] [2] =
+            gfc->bitrate_blockType_Hist [i] [3] =
+            gfc->bitrate_blockType_Hist [i] [4] =
+            gfc->bitrate_blockType_Hist [i] [5] = 0;
+    }
+    
     /* Write initial VBR Header to bitstream and init VBR data */
     if (gfp->bWriteVbrTag)
         InitVbrTag(gfp);
@@ -1962,6 +2055,11 @@ lame_close(lame_global_flags * gfp)
     if (gfc->Class_ID != LAME_ID)
         return -3;
 
+    if (gfp->exp_nspsytune2.pointer[0]) {
+      fclose((FILE *)gfp->exp_nspsytune2.pointer[0]);
+      gfp->exp_nspsytune2.pointer[0] = NULL;
+    }
+
     gfc->Class_ID = 0;
 
     // this routien will free all malloc'd data in gfc, and then free gfc:
@@ -1969,8 +2067,10 @@ lame_close(lame_global_flags * gfp)
 
     gfp->internal_flags = NULL;
 
-    if (gfp->lame_allocated_gfp)
+    if (gfp->lame_allocated_gfp) {
+        gfp->lame_allocated_gfp = 0;
         free(gfp);
+    }
 
     return 0;
 }
@@ -2024,6 +2124,9 @@ lame_init(void)
     lame_global_flags *gfp;
     int     ret;
 
+
+    init_log_table();
+
     gfp = calloc(1, sizeof(lame_global_flags));
     if (gfp == NULL)
         return NULL;
@@ -2104,6 +2207,7 @@ lame_init_old(lame_global_flags * gfp)
                                 /* 2 = equal loudness curve */
     gfp->athaa_sensitivity = 0.0; /* no offset */
     gfp->useTemporal = -1;
+    gfp->interChRatio = 0.0;
 
     /* The reason for
      *       int mf_samples_to_encode = ENCDELAY + POSTDELAY;
@@ -2126,7 +2230,11 @@ lame_init_old(lame_global_flags * gfp)
 #ifdef DECODE_ON_THE_FLY
     lame_decode_init()  // initialize the decoder 
 #endif
+    gfp->asm_optimizations.mmx = 1;
+    gfp->asm_optimizations.amd3dnow = 1;
+    gfp->asm_optimizations.sse = 1;
 
+    gfp->preset = 0;
     return 0;
 }
 
@@ -2212,10 +2320,7 @@ lame_stereo_mode_hist(const lame_global_flags * const gfp, int stmode_count[4])
         return;
 
     for (i = 0; i < 4; i++) {
-        int     j, sum = 0;
-        for (j = 0; j < 14; j++)
-            sum += gfc->bitrate_stereoMode_Hist[j + 1][i];
-        stmode_count[i] = sum;
+        stmode_count[i] = gfc->bitrate_stereoMode_Hist[15][i];
     }
 }
 
@@ -2242,4 +2347,47 @@ lame_bitrate_stereo_mode_hist(const lame_global_flags * const gfp,
             bitrate_stmode_count[j][i] = gfc->bitrate_stereoMode_Hist[j + 1][i];
 }
 
+
+
+void
+lame_block_type_hist(const lame_global_flags * const gfp, int btype_count[6])
+{
+    const lame_internal_flags *gfc;
+    int     i;
+
+    if (NULL == btype_count)
+        return;
+    if (NULL == gfp)
+        return;
+    gfc = gfp->internal_flags;
+    if (NULL == gfc)
+        return;
+
+    for (i = 0; i < 6; ++i) {
+        btype_count[i] = gfc->bitrate_blockType_Hist[15][i];
+    }
+}
+
+
+
+void 
+lame_bitrate_block_type_hist(const lame_global_flags * const gfp, 
+                             int bitrate_btype_count[14][6])
+{
+    const lame_internal_flags * gfc;
+    int     i, j;
+    
+    if (NULL == bitrate_btype_count)
+        return;
+    if (NULL == gfp)
+        return;
+    gfc = gfp->internal_flags;
+    if (NULL == gfc)
+        return;
+        
+    for (j = 0; j < 14; ++j)
+    for (i = 0; i <  6; ++i)
+        bitrate_btype_count[j][i] = gfc->bitrate_blockType_Hist[j+1][i];
+}
+
 /* end of lame.c */
