@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: psymodel.c,v 1.108 2002/03/03 18:57:16 aleidinger Exp $ */
+/* $Id: psymodel.c,v 1.114 2002/11/23 18:54:06 bouvigne Exp $ */
 
 
 /*
@@ -186,7 +186,7 @@ blocktype_d[2]        block type to use for previous granule
 #endif
 
 FLOAT
-psycho_loudness_approx( FLOAT *energy, lame_global_flags *gfp );
+psycho_loudness_approx( FLOAT *energy, lame_internal_flags *gfc );
 
 #ifdef NON_LINEAR_PSY
 
@@ -279,11 +279,6 @@ int L3psycho_anal( lame_global_flags * gfp,
       psymodel_init(gfp);
       init_fft(gfc);
       gfc->psymodel_init=1;
-      
-      for (chn = 0; chn < 4; ++chn )
-      for (b = 0; b < CBANDS; ++b ) 
-        gfc->nb_s1[chn][b] = gfc->nb_s2[chn][b] = 1.0;
-      
   }
   
 
@@ -361,7 +356,7 @@ int L3psycho_anal( lame_global_flags * gfp,
     gfc->energy[0] *= gfc->energy[0];
     
     /* sum total energy at nearly no  extra cost */
-    gfc->tot_ener[chn] = NON_LINEAR_SCALE_ENERGY(gfc->energy[0]);
+    gfc->tot_ener[chn] = 0.0;
     
     for (j=BLKSIZE/2-1; j >= 0; --j)
     {
@@ -402,7 +397,7 @@ int L3psycho_anal( lame_global_flags * gfp,
       if( chn < 2 ) {		/* no loudness for mid and side channels */
 	gfc->loudness_sq[gr_out][chn] = gfc->loudness_sq_save[chn];
 	gfc->loudness_sq_save[chn]
-	  = psycho_loudness_approx( gfc->energy, gfp);
+	  = psycho_loudness_approx( gfc->energy, gfc);
       }
     }
 
@@ -821,21 +816,13 @@ int L3psycho_anal( lame_global_flags * gfp,
 	      }
 	      ecb *= gfc->SNR_s[b];
 
-/* 2001-07-13 */
-        if ( gfp->VBR == vbr_off || gfp->VBR == vbr_abr ) {
-            /* this looks like a BUG to me. robert */
-            thr[b] = Max (1e-6, ecb);
-        }
-        else {
-            thr[b] = Min( ecb, rpelev_s  * gfc->nb_s1[chn][b] );
-            if (gfc->blocktype_old[chn>1 ? chn-2 : chn] == SHORT_TYPE ) {
-                thr[b] = Min( thr[b], rpelev2_s * gfc->nb_s2[chn][b] );
-            }
-            thr[b] = Max( thr[b], 1e-37 );
-            gfc->nb_s2[chn][b] = gfc->nb_s1[chn][b];
-            gfc->nb_s1[chn][b] = ecb;
-        }
-        
+	      thr[b] = Min( ecb, rpelev_s  * gfc->nb_s1[chn][b] );
+	      if (gfc->blocktype_old[chn>1 ? chn-2 : chn] == SHORT_TYPE ) {
+		  thr[b] = Min( thr[b], rpelev2_s * gfc->nb_s2[chn][b] );
+	      }
+	      thr[b] = Max( thr[b], 1e-37 );
+	      gfc->nb_s2[chn][b] = gfc->nb_s1[chn][b];
+	      gfc->nb_s1[chn][b] = ecb;
 	    }
 
 	  for ( sb = 0; sb < NBPSY_s; sb++ )
@@ -1088,6 +1075,55 @@ int L3psycho_anal( lame_global_flags * gfp,
   return 0;
 }
 
+
+
+/* mask_add optimization */
+/* init the limit values used to avoid computing log in mask_add when it is not necessary */
+
+/* For example, with i = 10*log10(m2/m1)/10*16         (= log10(m2/m1)*16)
+ *
+ * abs(i)>8 is equivalent (as i is an integer) to
+ * abs(i)>=9
+ * i>=9 || i<=-9
+ * equivalent to (as i is the biggest integer smaller than log10(m2/m1)*16 
+ * or the smallest integer bigger than log10(m2/m1)*16 depending on the sign of log10(m2/m1)*16)
+ * log10(m2/m1)>=9/16 || log10(m2/m1)<=-9/16
+ * exp10 is strictly increasing thus this is equivalent to
+ * m2/m1 >= 10^(9/16) || m2/m1<=10^(-9/16) which are comparisons to constants
+ */
+
+
+#define I1LIMIT 8   /* as in if(i>8)  */ 
+#define I2LIMIT 24  /* as in if(i>24) */ 
+#define MLIMIT  15  /* as in if(m<15) */ 
+
+static FLOAT8 ma_max_i1;
+static FLOAT8 ma_min_i1;
+static FLOAT8 ma_max_i2;
+static FLOAT8 ma_min_i2;
+static FLOAT8 ma_max_m;
+
+
+
+void init_mask_add_max_values(void)
+{
+  static int init=0;
+
+  if(!init) {
+    ma_max_i1 = pow(10,(I1LIMIT+1)/16.0);
+    ma_min_i1 = pow(10,-(I1LIMIT+1)/16.0);
+    ma_max_i2 = pow(10,(I2LIMIT+1)/16.0);
+    ma_min_i2 = pow(10,-(I2LIMIT+1)/16.0);
+    ma_max_m  = pow(10,(MLIMIT)/10.0);
+    init = 1;
+  }
+}
+
+
+
+
+
+
 /* addition of simultaneous masking   Naoki Shibata 2000/7 */
 inline static FLOAT8 mask_add(FLOAT8 m1,FLOAT8 m2,int k,int b, lame_internal_flags * const gfc)
 {
@@ -1111,37 +1147,100 @@ inline static FLOAT8 mask_add(FLOAT8 m1,FLOAT8 m2,int k,int b, lame_internal_fla
 
   int i;
   FLOAT8 m;
+  FLOAT ratio;
 
   if (m1 == 0) return m2;
 
+  ratio = m2/m1;
+
   if (b < 0) b = -b;
 
-  i = 10*log10(m2 / m1)/10*16;
-  m = 10*log10((m1+m2)/gfc->ATH->cb[k]);
+  /*i = abs(10*log10(m2 / m1)/10*16);
+  m = 10*log10((m1+m2)/gfc->ATH->cb[k]);*/
+
+
+  /* Should always be true, just checking */
+  assert(m1>=0);
+  assert(m2>=0);
+  assert(gfc->ATH->cb[k]>=0);
 
-  if (i < 0) i = -i;
 
   if (b <= 3) {  /* approximately, 1 bark = 3 partitions */
-    if (i > 8) return m1+m2;
+    /* 65% of the cases */
+
+    /* originally 'if(i > 8)' */
+    if(ratio>=ma_max_i1 || ratio <=ma_min_i1) {
+      /* 43% of the total */
+      return m1+m2;
+    }
+
+    /* 22% of the total */
+    i = fabs(FAST_LOG10(ratio))*16;
+    if (i > I1LIMIT)
+      return m1+m2; /* just in case  8.99999 <= log10(m2/m1)*16 < 9 */
+
     return (m1+m2)*table2[i];
   }
 
-  if (m<15) {
-    if (m > 0) {
+
+  /* m<15 equ log10((m1+m2)/gfc->ATH->cb[k])<1.5
+   * equ (m1+m2)/gfc->ATH->cb[k]<10^1.5
+   * equ (m1+m2)<10^1.5 * gfc->ATH->cb[k]
+   */
+
+  if((m1+m2)<ma_max_m*gfc->ATH->cb[k])  {
+    /* 3% of the total */
+    i = fabs(FAST_LOG10(ratio))*16;
+
+    /* Originally if (m > 0) { */
+    if(m1+m2>gfc->ATH->cb[k]) {
       FLOAT8 f=1.0,r;
+
       if (i > 24) return m1+m2;
+
       if (i > 13) f = 1; else f = table3[i];
+
+      m = 10*FAST_LOG10((m1+m2)/gfc->ATH->cb[k]);
       r = (m-0)/15;
+
       return (m1+m2)*(table1[i]*r+f*(1-r));
     }
+
     if (i > 13) return m1+m2;
+
     return (m1+m2)*table3[i];
   }
 
-  if (i > 24) return m1+m2;
+
+  /* orginally 'if (i > 24) {' */
+  if(ratio>=ma_max_i2 || ratio <=ma_min_i2) {
+    /* 22% of total */
+    return m1+m2;
+  }
+
+  
+  /* 10% of total */
+  i = fabs(FAST_LOG10(ratio))*16;
+  if (i > I2LIMIT)
+    return m1+m2; /* just in case... */
+
   return (m1+m2)*table1[i];
 }
 
+
+
+inline FLOAT8 NS_INTERP(FLOAT8 x, FLOAT8 y, FLOAT8 r)
+{
+    /* was pow((x),(r))*pow((y),1-(r))*/
+    if(r==1.0)
+        return x;              /* 99.7% of the time */ 
+    if(y!=0.0)
+        return pow(x/y,r)*y;   /* rest of the time */ 
+    return 0.0;                /* never happens */ 
+}
+
+
+
 int L3psycho_anal_ns( lame_global_flags * gfp,
                     const sample_t *buffer[2], int gr_out, 
                     FLOAT8 *ms_ratio,
@@ -1182,7 +1281,7 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 
   /* variables used for --nspsytune */
   int ns_attacks[4];
-  FLOAT ns_hpfsmpl[4][576+576/3+NSFIRLEN];
+  FLOAT ns_hpfsmpl[4][576];
   FLOAT pe_l[4],pe_s[4];
   FLOAT pcfact;
 
@@ -1219,28 +1318,36 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
       -8.65163e-18,
     };
 
-    for(chn=0;chn<gfc->channels_out;chn++)
-      {
-	FLOAT firbuf[576+576/3+NSFIRLEN];
+    /* Don't copy the input buffer into a temporary buffer */
+    /* unroll the loop 4 times */
+    for(chn=0;chn<gfc->channels_out;chn++) {
+      
+      /* apply high pass filter of fs/4 */
+      const sample_t * const firbuf = &buffer[chn][576-350-NSFIRLEN+576/3];
 
-	/* apply high pass filter of fs/4 */
-	
-	for(i=-NSFIRLEN;i<576+576/3;i++)
-	  firbuf[i+NSFIRLEN] = buffer[chn][576-350+(i)];
+      for(i=0;i<576;i++) {
+        FLOAT sum1 = 0, sum2=0, sum3=0, sum4=0;
+
+        /* align on 4 */
+        for(j=0;j<(NSFIRLEN%4);j++)
+          sum1 += fircoef[j] * firbuf[i+j];
+
+        /* aligned on 4 */
+        for(;j<NSFIRLEN;j+=4) {
+          sum1 += fircoef[j] * firbuf[i+j];
+          sum2 += fircoef[j+1] * firbuf[i+j+1];
+          sum3 += fircoef[j+2] * firbuf[i+j+2];
+          sum4 += fircoef[j+3] * firbuf[i+j+3];
+        }
+
+        ns_hpfsmpl[chn][i] = sum1+sum2+sum3+sum4;
 
-	for(i=0;i<576+576/3-NSFIRLEN;i++)
-          {
-	    FLOAT sum = 0;
-	    for(j=0;j<NSFIRLEN;j++)
-	      sum += fircoef[j] * firbuf[i+j];
-	    ns_hpfsmpl[chn][i] = sum;
-	  }
-	for(;i<576+576/3;i++)
-	  ns_hpfsmpl[chn][i] = 0;
       }
+    }
+
 
     if (gfp->mode == JOINT_STEREO) {
-      for(i=0;i<576+576/3;i++)
+      for(i=0;i<576;i++)
 	{
 	  ns_hpfsmpl[2][i] = ns_hpfsmpl[0][i]+ns_hpfsmpl[1][i];
 	  ns_hpfsmpl[3][i] = ns_hpfsmpl[0][i]-ns_hpfsmpl[1][i];
@@ -1281,6 +1388,10 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
   }
 
   for (chn=0; chn<numchn; chn++) {
+      FLOAT en_subshort[12];
+      FLOAT attack_intensity[12];
+      int ns_uselongblock = 1;
+      FLOAT attackThreshold;
 
     /**********************************************************************
      *  compute FFTs
@@ -1328,7 +1439,7 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
     gfc->energy[0] *= gfc->energy[0];
     
     /* sum total energy at nearly no extra cost */
-    gfc->tot_ener[chn] = gfc->energy[0];
+    gfc->tot_ener[chn] = 0.0;
     
     for (j=BLKSIZE/2-1; j >= 0; --j)
     {
@@ -1375,7 +1486,7 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
       if( chn < 2 ) {		/* no loudness for mid and side channels */
 	gfc->loudness_sq[gr_out][chn] = gfc->loudness_sq_save[chn];
 	gfc->loudness_sq_save[chn]
-	  = psycho_loudness_approx( gfc->energy, gfp);
+	  = psycho_loudness_approx( gfc->energy, gfc);
       }
     }
 
@@ -1423,8 +1534,8 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	    }
 	  }
  
-	a /= c1;
-	tonality2[b] = a == 0 ? 0 : (m / a - 1)/(c2-1);
+  if(a) tonality2[b] = (m*c1-a) / (a*(c2-1));
+    else  tonality2[b] = 0;
       }
 
     if (gfc->nsPsy.use2) {
@@ -1498,15 +1609,20 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	ecb = 0;
 
 #if 1
-	for ( k = gfc->s3ind[b][0]; k <= gfc->s3ind[b][1]; k++ )
-	  {
+	k = gfc->s3ind[b][0]; 
+    ecb = gfc->s3_ll[kk++] * eb2[k];
+
+    for ( k = k+1; k <= gfc->s3ind[b][1]; k++ ) 
+    {
 	    ecb = mask_add(ecb,gfc->s3_ll[kk++] * eb2[k],k,k-b,gfc);
-	  }
+    }
 
 	ecb *= 0.158489319246111; // pow(10,-0.8)
-#endif
 
-#if 0
+
+#else
+    ecb = 0;
+
 	for ( k = gfc->s3ind[b][0]; k <= gfc->s3ind[b][1]; k++ )
 	  {
 	    ecb += gfc->s3_ll[kk++] * eb2[k];
@@ -1529,7 +1645,6 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	   chn=2,3   S and M channels.  
 	*/
 
-#define NS_INTERP(x,y,r) (pow((x),(r))*pow((y),1-(r)))
 
 	if (gfc->blocktype_old[chn>1 ? chn-2 : chn] == SHORT_TYPE )
 	  thr[b] = ecb; /* Min(ecb, rpelev*gfc->nb_1[chn][b]); */
@@ -1545,78 +1660,65 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
      * determine the block type (window type)
      ***************************************************************/
 
-    {
-      static int count=0;
-      FLOAT en_subshort[12];
-      FLOAT attack_intensity[12];
-      int ns_uselongblock = 1;
+	/* calculate energies of each sub-shortblocks */
+	for (i=0; i<3; i++) {
+	    en_subshort[i] = gfc->nsPsy.last_en_subshort[chn][i+6];
+	    attack_intensity[i]
+		= en_subshort[i] / gfc->nsPsy.last_en_subshort[chn][i+4];
+	}
 
-      /* calculate energies of each sub-shortblocks */
-
-      k = 0;
-      for(i=0;i<12;i++)
-	{
-	  en_subshort[i] = 0;
-	  for(j=0;j<576/9;j++)
-	    {
-	      en_subshort[i] += ns_hpfsmpl[chn][k] * ns_hpfsmpl[chn][k];
-	      k++;
+	if (chn == 2) {
+	    for(i=0;i<576;i++) {
+		FLOAT l, r;
+		l = ns_hpfsmpl[0][i];
+		r = ns_hpfsmpl[1][i];
+		ns_hpfsmpl[0][i] = l+r;
+		ns_hpfsmpl[1][i] = l-r;
 	    }
-
-	  if (en_subshort[i] < 100) en_subshort[i] = 100;
 	}
-
-      /* compare energies between sub-shortblocks */
-
-#define NSATTACKTHRE 150
-#define NSATTACKTHRE_S 300
-
-      for(i=0;i<2;i++) {
-	attack_intensity[i] = en_subshort[i] / gfc->nsPsy.last_en_subshort[chn][7+i];
-      }
-
-      for(;i<12;i++) {
-	attack_intensity[i] = en_subshort[i] / en_subshort[i-2];
-      }
-
-      ns_attacks[0] = ns_attacks[1] = ns_attacks[2] = ns_attacks[3] = 0;
-
-      for(i=0;i<12;i++)
 	{
-	  if (!ns_attacks[i/3] && attack_intensity[i] > (chn == 3 ? (gfc->presetTune.use ? gfc->presetTune.attackthre_s : NSATTACKTHRE_S)
-                                                              : (gfc->presetTune.use ? gfc->presetTune.attackthre   : NSATTACKTHRE))) ns_attacks[i/3] = (i % 3)+1;
+	    FLOAT *pf = ns_hpfsmpl[chn & 1];
+	    for (i=0;i<9;i++) {
+		double p = 0.0;
+		FLOAT *pfe = pf + 576/9;
+		for (; pf < pfe; pf++)
+		    p += *pf * *pf;
+
+		if (p < 100.0)
+		    p = 100.0;
+		gfc->nsPsy.last_en_subshort[chn][i] = en_subshort[i+3] = p;
+		attack_intensity[i+3] = p / en_subshort[i+3-2];
+	    }
 	}
 
-      if (ns_attacks[0] && gfc->nsPsy.last_attacks[chn][2]) ns_attacks[0] = 0;
-      if (ns_attacks[1] && ns_attacks[0]) ns_attacks[1] = 0;
-      if (ns_attacks[2] && ns_attacks[1]) ns_attacks[2] = 0;
-      if (ns_attacks[3] && ns_attacks[2]) ns_attacks[3] = 0;
+	/* compare energies between sub-shortblocks */
+	ns_attacks[0] = ns_attacks[1] = ns_attacks[2] = ns_attacks[3] = 0;
+	attackThreshold = (chn == 3)
+	    ? gfc->presetTune.attackthre_s : gfc->presetTune.attackthre;
+	for (i=0;i<12;i++) {
+	    if (ns_attacks[i/3])
+		continue;
+	
+	    if (attack_intensity[i] > attackThreshold)
+		ns_attacks[i/3] = (i % 3)+1;
+	}
+        if (ns_attacks[0] && gfc->nsPsy.last_attacks[chn][2])
+	    ns_attacks[0] = 0;
 
-      if (gfc->nsPsy.last_attacks[chn][2] == 3 ||
-	  ns_attacks[0] || ns_attacks[1] || ns_attacks[2] || ns_attacks[3]) ns_uselongblock = 0;
+	if (gfc->nsPsy.last_attacks[chn][2] == 3 ||
+	    ns_attacks[0] || ns_attacks[1] || ns_attacks[2] || ns_attacks[3]) {
+	    ns_uselongblock = 0;
 
-      if (chn < 4) count++;
-
-      for(i=0;i<9;i++)
-	{
-	  gfc->nsPsy.last_en_subshort[chn][i] = en_subshort[i];
-	  gfc->nsPsy.last_attack_intensity[chn][i] = attack_intensity[i];
+	    if (ns_attacks[1] && ns_attacks[0]) ns_attacks[1] = 0;
+	    if (ns_attacks[2] && ns_attacks[1]) ns_attacks[2] = 0;
+	    if (ns_attacks[3] && ns_attacks[2]) ns_attacks[3] = 0;
 	}
 
-      if (gfp->short_blocks == short_block_dispensed) {
-	uselongblock[chn] = 1;
-      } 
-      else if (gfp->short_blocks == short_block_forced) {
-	uselongblock[chn] = 0;
-      }
-      else {
 	if (chn < 2) {
-	  uselongblock[chn] = ns_uselongblock;
+	    uselongblock[chn] = ns_uselongblock;
 	} else {
-	  if (ns_uselongblock == 0) uselongblock[0] = uselongblock[1] = 0;
+	    if (!ns_uselongblock) uselongblock[0] = uselongblock[1] = 0;
 	}
-      }
-    }
 
 #if defined(HAVE_GTK)
     if (gfp->analysis) {
@@ -1687,10 +1789,9 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 		}
 
 /* 2001-07-13 */
-            /* this looks like a BUG  */
-            thr[b] = Max (1e-6, ecb);
-            
-            if (gfp->VBR == vbr_mtrh) {
+          thr[b] = Max(1e-6, ecb);
+
+          if (gfp->VBR == vbr_mtrh) {
                 thr[b] = Min( ecb, rpelev_s  * gfc->nb_s1[chn][b] );
                 if (gfc->blocktype_old[chn>1 ? chn-2 : chn] == SHORT_TYPE ) {
                    thr[b] = Min( thr[b], rpelev2_s * gfc->nb_s2[chn][b] );
@@ -1698,7 +1799,7 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
                 thr[b] = Max( thr[b], 1e-37 );
                 gfc->nb_s2[chn][b] = gfc->nb_s1[chn][b];
                 gfc->nb_s1[chn][b] = ecb;
-                }
+          }
 	    }
 	}
 	for ( sb = 0; sb < NBPSY_s; sb++ )
@@ -1982,6 +2083,14 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	uselongblock[1]=0;
       }
   }
+  else if (gfp->short_blocks == short_block_dispensed) {
+    uselongblock[0]=1;
+    uselongblock[1]=1;
+  }
+  else if (gfp->short_blocks == short_block_forced) {
+    uselongblock[0]=0;
+    uselongblock[1]=0;
+  } 
 
   /* update the blocktype of the previous granule, since it depends on what
    * happend in this granule */
@@ -2168,6 +2277,7 @@ int *npart_l_orig,int *npart_l,int *npart_s_orig,int *npart_s)
       bo_l[sfb]=partition[i2];
 
     }
+    w1_l[0] = w2_l[sfb-1] = 1.0;
   }
 
 
@@ -2277,6 +2387,7 @@ int *npart_l_orig,int *npart_l,int *npart_s_orig,int *npart_s)
       bo_s[sfb]=partition[i2];
 
     }
+    w1_s[0] = w2_s[sfb-1] = 1.0;
   }
 
 
@@ -2370,6 +2481,25 @@ int *npart_l_orig,int *npart_l,int *npart_s_orig,int *npart_s)
     /* setup temporal masking */
     gfc->decay = exp(-1.0*LOG10/(temporalmask_sustain_sec*sfreq/192.0));
 
+    if (gfp->ATHtype != -1) {
+        /* compute equal loudness weights (eql_w) */
+        FLOAT freq = 0.0;
+        FLOAT freq_inc = gfp->out_samplerate / (BLKSIZE);
+        FLOAT eql_balance = 0.0;
+ 
+        for (i = 0; i < BLKSIZE/2; ++i) {
+            /* convert ATH dB to relative power (not dB) to determine eql_w */
+            freq += freq_inc;
+            gfc->ATH->eql_w[i] = 1. / pow( 10, ATHformula( freq, gfp ) / 10 );
+            eql_balance += gfc->ATH->eql_w[i];
+        }
+        eql_balance = 1.0 / eql_balance;
+
+        for (i = BLKSIZE/2; --i >= 0;) { /* scale weights */
+            gfc->ATH->eql_w[i] *= eql_balance;
+        }
+    }
+    
     return 0;
 }
 
@@ -2392,6 +2522,9 @@ int psymodel_init(lame_global_flags *gfp)
     FLOAT8 s3_l[CBANDS][CBANDS];
     int numberOfNoneZero;
 
+
+    init_mask_add_max_values();
+
     samplerate = gfp->out_samplerate;
     gfc->ms_ener_ratio_old=.25;
     gfc->blocktype_old[0]=STOP_TYPE;
@@ -2403,6 +2536,7 @@ int psymodel_init(lame_global_flags *gfp)
       for (j=0; j<CBANDS; ++j) {
 	gfc->nb_1[i][j]=1e20;
 	gfc->nb_2[i][j]=1e20;
+        gfc->nb_s1[i][j] = gfc->nb_s2[i][j] = 1.0;
       }
       for ( sb = 0; sb < NBPSY_l; sb++ ) {
 	gfc->en[i].l[sb] = 1e20;
@@ -2564,6 +2698,17 @@ int psymodel_init(lame_global_flags *gfp)
 
 
 
+    if (gfc->nsPsy.use) {
+
+#define NSATTACKTHRE 150
+#define NSATTACKTHRE_S 300
+
+	if (!gfc->presetTune.use) {
+	    gfc->presetTune.attackthre_s = NSATTACKTHRE_S;
+	    gfc->presetTune.attackthre   = NSATTACKTHRE;
+	}
+    }
+
     return 0;
 }
 
@@ -2588,42 +2733,23 @@ future:  Data indicates that the shape of the equal loudness curve varies
          However, the potential gain may not be enough to justify an effort.
 */
 FLOAT
-psycho_loudness_approx( FLOAT *energy, lame_global_flags *gfp )
+psycho_loudness_approx( FLOAT *energy, lame_internal_flags *gfc )
 {
-  int i;
-  static int eql_type = -1;
-  static FLOAT eql_w[BLKSIZE/2];/* equal loudness weights (based on ATH) */
-  const FLOAT vo_scale= 1./( 14752 ); /* tuned for output level */
+  const FLOAT vo_scale= 1.f/( 14752 ); /* tuned for output level */
 				      /* (sensitive to energy scale) */
-  FLOAT loudness_power;
+  const FLOAT w = 1.f/(BLKSIZE/2);
+  const FLOAT scale = vo_scale * vo_scale * w;
 
-  if( eql_type != gfp->ATHtype ) { 
-				/* compute equal loudness weights (eql_w) */
-    FLOAT freq;
-    FLOAT freq_inc = gfp->out_samplerate / (BLKSIZE);
-    FLOAT eql_balance = 0.0;
-    eql_type = gfp->ATHtype;
-    freq = 0.0;
-    for( i = 0; i < BLKSIZE/2; ++i ) {
-      freq += freq_inc;
-				/* convert ATH dB to relative power (not dB) */
-				/*  to determine eql_w */
-      eql_w[i] = 1. / pow( 10, ATHformula( freq, gfp ) / 10 );
-      eql_balance += eql_w[i];
-    }
-    eql_balance = 1 / eql_balance;
-    for( i = BLKSIZE/2; --i >= 0; ) { /* scale weights */
-      eql_w[i] *= eql_balance;
-    }
-  }
+  const FLOAT* eql_w = gfc->ATH->eql_w;/* equal loudness weights (based on ATH) */
+  FLOAT loudness_power = 0.0;
+  int i;
 
-  loudness_power = 0.0;
-  for( i = 0; i < BLKSIZE/2; ++i ) { /* apply weights to power in freq. bands*/
+  for (i = 0; i < BLKSIZE/2; ++i) {
+    /* apply weights to power in freq. bands*/
     loudness_power += NON_LINEAR_SCALE_ITEM(energy[i]) * eql_w[i];
   }
   loudness_power = NON_LINEAR_SCALE_SUM(loudness_power);
-  loudness_power /= (BLKSIZE/2);
-  loudness_power *= vo_scale * vo_scale;
+  loudness_power *= scale;
 
-  return( loudness_power );
+  return loudness_power;
 }
