@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: quantize.c,v 1.80 2001/12/18 16:14:08 dibrom Exp $ */
+/* $Id: quantize.c,v 1.121 2002/09/30 16:49:30 bouvigne Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -40,6 +40,22 @@
 #endif
 
 
+/* convert from L/R <-> Mid/Side, src == dst allowed */
+inline static void 
+ms_convert(III_side_info_t *l3_side, int gr)
+{
+    int i;
+    for (i = 0; i < 576; ++i) {
+	FLOAT8 l, r;
+        l = l3_side->tt[gr][0].xr[i];
+        r = l3_side->tt[gr][1].xr[i];
+        l3_side->tt[gr][0].xr[i] = (l+r) * (FLOAT8)(SQRT2*0.5);
+        l3_side->tt[gr][1].xr[i] = (l-r) * (FLOAT8)(SQRT2*0.5);
+    }
+}
+
+
+
 /************************************************************************
  *
  *      init_outer_loop()
@@ -52,16 +68,38 @@
  ************************************************************************/
 
 static int 
-init_outer_loop(
+init_xrpow(
     lame_internal_flags *gfc,
     gr_info *const cod_info, 
-    III_scalefac_t *const scalefac, 
-    const FLOAT8 xr[576], 
     FLOAT8 xrpow[576] )
 {
     FLOAT8 tmp, sum = 0;
     int i;
+    memset(&gfc->pseudohalf, 0, sizeof(gfc->pseudohalf));
+    assert( xrpow != NULL );
+    
+    /*  check if there is some energy we have to quantize
+     *  and calculate xrpow matching our fresh scalefactors
+     */
+    for (i = 0; i < 576; ++i) {
+        tmp = fabs (cod_info->xr[i]);
+	sum += tmp;
+        xrpow[i] = sqrt (tmp * sqrt(tmp));
+    }
+    /*  return 1 if we have something to quantize, else 0
+     */
+    if (sum > (FLOAT8)1E-20)
+      return 1;
 
+    memset(&cod_info->l3_enc, 0, sizeof(int)*576);
+    return 0;
+}
+
+static void
+init_outer_loop(
+    lame_internal_flags *gfc,
+    gr_info *const cod_info)
+{
     /*  initialize fresh cod_info
      */
     cod_info->part2_3_length      = 0;
@@ -69,7 +107,6 @@ init_outer_loop(
     cod_info->count1              = 0;
     cod_info->global_gain         = 210;
     cod_info->scalefac_compress   = 0;
-    /* window_switching_flag was set in psymodel.c? */
     /* block_type            was set in psymodel.c? */
     /* mixed_block_flag      would be set in ^      */
     cod_info->table_select [0]    = 0;
@@ -84,21 +121,48 @@ init_outer_loop(
     cod_info->scalefac_scale      = 0;
     cod_info->count1table_select  = 0;
     cod_info->part2_length        = 0;
+    cod_info->sfb_lmax        = SBPSY_l;
+    cod_info->sfb_smin        = SBPSY_s;
+    cod_info->psy_lmax        = gfc->sfb21_extra ? SBMAX_l : SBPSY_l;
+    cod_info->psy_smax        = 0;
     if (cod_info->block_type == SHORT_TYPE) {
-        cod_info->sfb_lmax        = 0;
+	int sfb;
+	FLOAT8 ixwork[576];
+	FLOAT8 *ix;
+
         cod_info->sfb_smin        = 0;
+        cod_info->psy_smax        = gfc->sfb21_extra ? SBMAX_s : SBPSY_s;
+        cod_info->sfb_lmax        = 0;
 	if (cod_info->mixed_block_flag) {
             /*
              *  MPEG-1:      sfbs 0-7 long block, 3-12 short blocks 
              *  MPEG-2(.5):  sfbs 0-5 long block, 3-12 short blocks
              */ 
-            cod_info->sfb_lmax    = gfc->is_mpeg1 ? 8 : 6;
 	    cod_info->sfb_smin    = 3;
+            cod_info->sfb_lmax    = gfc->is_mpeg1 ? 8 : 6;
 	}
-    } else {
-        cod_info->sfb_lmax        = SBPSY_l;
-        cod_info->sfb_smin        = SBPSY_s;
-    }   
+	cod_info->psy_lmax    = cod_info->sfb_lmax;
+	/* re-order the short blocks, for more efficient encoding below */
+	/* By Takehiro TOMINAGA */
+	/*
+	  Within each scalefactor band, data is given for successive
+	  time windows, beginning with window 0 and ending with window 2.
+	  Within each window, the quantized values are then arranged in
+	  order of increasing frequency...
+	*/
+	ix = &cod_info->xr[gfc->scalefac_band.l[cod_info->sfb_lmax]];
+	memcpy(ixwork, cod_info->xr, 576*sizeof(FLOAT8));
+	for (sfb = cod_info->sfb_smin; sfb < SBMAX_s; sfb++) {
+	    int start = gfc->scalefac_band.s[sfb];
+	    int end   = gfc->scalefac_band.s[sfb + 1];
+	    int window, l;
+	    for (window = 0; window < 3; window++) {
+		for (l = start; l < end; l++) {
+		    *ix++ = ixwork[3*l+window];
+		}
+	    }
+	}
+    }
     cod_info->count1bits          = 0;  
     cod_info->sfb_partition_table = nr_of_sfb_block[0][0];
     cod_info->slen[0]             = 0;
@@ -108,20 +172,7 @@ init_outer_loop(
 
     /*  fresh scalefactors are all zero
      */
-    memset(scalefac, 0, sizeof(III_scalefac_t));
-    memset(&gfc->pseudohalf, 0, sizeof(gfc->pseudohalf));
-
-    /*  check if there is some energy we have to quantize
-     *  and calculate xrpow matching our fresh scalefactors
-     */
-    for (i = 0; i < 576; ++i) {
-        tmp = fabs (xr[i]);
-	sum += tmp;
-        xrpow[i] = sqrt (tmp * sqrt(tmp));
-    }
-   /*  return 1 if we have something to quantize, else 0
-    */
-   return sum > (FLOAT8)1E-20;
+    memset(&cod_info->scalefac, 0, sizeof(III_scalefac_t));
 }
 
 
@@ -143,7 +194,8 @@ typedef enum {
     BINSEARCH_DOWN
 } binsearchDirection_t;
 
-int 
+inline
+static int 
 bin_search_StepSize(
           lame_internal_flags * const gfc,
           gr_info * const cod_info,
@@ -212,7 +264,8 @@ bin_search_StepSize(
  *
  ***************************************************************************/ 
 
-int 
+inline
+static int 
 inner_loop(
           lame_internal_flags * const gfc,
           gr_info * const cod_info,
@@ -265,7 +318,7 @@ loop_break(
 
     for (sfb = cod_info->sfb_smin; sfb < SBPSY_s; sfb++)
         for (i = 0; i < 3; i++) 
-            if (scalefac->s[sfb][i] == 0 && cod_info->subblock_gain[i] == 0)
+            if (scalefac->s[sfb][i] + cod_info->subblock_gain[i] == 0)
                 return 0;
 
     return 1;
@@ -319,14 +372,19 @@ quant_compare(
         case 2: 
 	    better = calc->tot_noise < best->tot_noise; 
 	    break;
-        case 3: 
-		better = ( calc->tot_noise < (gfc->presetTune.use &&
-                                      block_type != NORM_TYPE ? (best->tot_noise - gfc->presetTune.quantcomp_adjust_rh_tot)
-                                                          :  best->tot_noise ) &&
-                   calc->max_noise < (gfc->presetTune.use &&
-                                      block_type != NORM_TYPE ? (best->max_noise - gfc->presetTune.quantcomp_adjust_rh_max)
-                                                          :  best->max_noise ));
-	    break;
+        case 3: {
+            better = (calc->tot_noise < best->tot_noise) &&
+                     (calc->max_noise < best->max_noise );
+            
+            if (gfc->presetTune.use) {
+                if ( block_type != NORM_TYPE) {
+                    better = (calc->tot_noise < (best->tot_noise - gfc->presetTune.quantcomp_adjust_rh_tot)) &&
+                             (calc->max_noise < (best->max_noise - gfc->presetTune.quantcomp_adjust_rh_max));
+                }            
+            }
+
+            break;
+        }
         case 4: 
 	    better = ( calc->max_noise <= 0  &&
                        best->max_noise >  2 )
@@ -365,7 +423,8 @@ quant_compare(
                    ||  calc->over_noise < best->over_noise; 
 	    break;
         case 8: 
-	    better =   calc->klemm_noise < best->klemm_noise;
+	    better =   Max(1e-20, calc->klemm_noise)
+	      < Max(1e-20, best->klemm_noise);
             break;
     }   
 
@@ -413,7 +472,7 @@ amp_scalefac_bands(
     FLOAT8 xrpow[576] )
 {
   lame_internal_flags *gfc=gfp->internal_flags;
-  int start, end, l,i,j,sfb;
+  int j, sfb;
   FLOAT8 ifqstep34, trigger;
 
   if (cod_info->scalefac_scale == 0) {
@@ -429,18 +488,17 @@ amp_scalefac_bands(
         trigger = distort->l[sfb];
   }
   for (sfb = cod_info->sfb_smin; sfb < SBPSY_s; sfb++) {
-    for (i = 0; i < 3; i++ ) {
-      if (trigger < distort->s[sfb][i])
-          trigger = distort->s[sfb][i];
-    }
+      int b;
+      for (b = 0; b < 3; b++ ) {
+	  if (trigger < distort->s[sfb][b])
+	      trigger = distort->s[sfb][b];
+      }
   }
 
   switch (gfc->noise_shaping_amp) {
 
-  case 3:
   case 2:
     /* amplify exactly 1 band */
-    //trigger = distort_thresh;
     break;
 
   case 1:
@@ -461,49 +519,47 @@ amp_scalefac_bands(
     break;
   }
 
+  j = 0;
   for (sfb = 0; sfb < cod_info->sfb_lmax; sfb++ ) {
-    start = gfc->scalefac_band.l[sfb];
-    end   = gfc->scalefac_band.l[sfb+1];
-    if (distort->l[sfb]>=trigger  ) {
-      if (gfc->noise_shaping_amp==3) {
-	if (gfc->pseudohalf.l[sfb]) {
-	  gfc->pseudohalf.l[sfb] = 0;
-	  goto done;
-	}
-	gfc->pseudohalf.l[sfb] = 1;
+      int width = gfc->scalefac_band.l[sfb+1] - gfc->scalefac_band.l[sfb];
+      int l;
+      j += width;
+      if (distort->l[sfb] < trigger)
+	  continue;
+
+      if (gfc->substep_shaping == 2) {
+	  gfc->pseudohalf.l[sfb] = !gfc->pseudohalf.l[sfb];
+	  if (!gfc->pseudohalf.l[sfb] && gfc->noise_shaping_amp==2)
+	      return;
       }
       scalefac->l[sfb]++;
-      for ( l = start; l < end; l++ )
-	xrpow[l] *= ifqstep34;
-      if (gfc->noise_shaping_amp==2
-	  ||gfc->noise_shaping_amp==3) goto done;
-    }
+      for (l = -width; l < 0; l++)
+	  xrpow[j+l] *= ifqstep34;
+      if (gfc->noise_shaping_amp==2)
+	  return;
   }
-  
-  for ( j=0,sfb = cod_info->sfb_smin; sfb < SBPSY_s; sfb++ ) {
-    start = gfc->scalefac_band.s[sfb];
-    end   = gfc->scalefac_band.s[sfb+1];
-    for ( i = 0; i < 3; i++ ) {
-      int j2 = j;
-      if ( distort->s[sfb][i]>=trigger) {
-	if (gfc->noise_shaping_amp==3) {
-	  if (gfc->pseudohalf.s[sfb][i]) {
-	    gfc->pseudohalf.s[sfb][i] = 0;
-	    goto done;
+
+  for (sfb = cod_info->sfb_smin; sfb < SBPSY_s; sfb++ ) {
+      int width = gfc->scalefac_band.s[sfb+1] - gfc->scalefac_band.s[sfb];
+      int b;
+      for (b = 0; b < 3; b++) {
+	  int l;
+	  j += width;
+	  if (distort->s[sfb][b] < trigger)
+	      continue;
+
+	  if (gfc->substep_shaping == 2) {
+	      gfc->pseudohalf.s[sfb][b] = !gfc->pseudohalf.s[sfb][b];
+	      if (!gfc->pseudohalf.s[sfb][b] && gfc->noise_shaping_amp==2)
+		  return;
 	  }
-	  gfc->pseudohalf.s[sfb][i] = 1;
-	}
-	scalefac->s[sfb][i]++;
-	for (l = start; l < end; l++) 
-	  xrpow[j2++] *= ifqstep34;
-	if (gfc->noise_shaping_amp==2
-	    ||gfc->noise_shaping_amp==3) goto done;
+	  scalefac->s[sfb][b]++;
+	  for (l = -width; l < 0; l++) 
+	      xrpow[j+l] *= ifqstep34;
+	  if (gfc->noise_shaping_amp==2)
+	      return;
       }
-      j += end-start;
-    }
   }
- done:
- return;
 }
 
 /*************************************************************************
@@ -523,36 +579,36 @@ inc_scalefac_scale (
           III_scalefac_t * const scalefac,
           FLOAT8                 xrpow[576] )
 {
-    int start, end, l,i,j;
-    int sfb;
+    int l, j, sfb;
     const FLOAT8 ifqstep34 = 1.29683955465100964055;
 
+    j = 0;
     for (sfb = 0; sfb < cod_info->sfb_lmax; sfb++) {
+	int width = gfc->scalefac_band.l[sfb+1] - gfc->scalefac_band.l[sfb];
         int s = scalefac->l[sfb] + (cod_info->preflag ? pretab[sfb] : 0);
+	j += width;
         if (s & 1) {
             s++;
-            start = gfc->scalefac_band.l[sfb];
-            end   = gfc->scalefac_band.l[sfb+1];
-            for (l = start; l < end; l++) 
-                xrpow[l] *= ifqstep34;
+            for (l = -width; l < 0; l++) 
+                xrpow[j+l] *= ifqstep34;
         }
         scalefac->l[sfb]  = s >> 1;
-        cod_info->preflag = 0;
     }
+    cod_info->preflag = 0;
 
-    for (j = 0, sfb = cod_info->sfb_smin; sfb < SBPSY_s; sfb++) {
-    start = gfc->scalefac_band.s[sfb];
-    end   = gfc->scalefac_band.s[sfb+1];
-    for (i = 0; i < 3; i++) {
-        int j2 = j;
-        if (scalefac->s[sfb][i] & 1) {
-        scalefac->s[sfb][i]++;
-        for (l = start; l < end; l++) 
-            xrpow[j2++] *= ifqstep34;
-        }
-        scalefac->s[sfb][i] >>= 1;
-        j += end-start;
-    }
+    for (sfb = cod_info->sfb_smin; sfb < SBPSY_s; sfb++) {
+	int width = gfc->scalefac_band.s[sfb+1] - gfc->scalefac_band.s[sfb];
+	int b;
+	for (b = 0; b < 3; b++) {
+	    int s = scalefac->s[sfb][b];
+	    j += width;
+	    if (s & 1) {
+		s++;
+		for (l = -width; l < 0; l++) 
+		    xrpow[j+l] *= ifqstep34;
+	    }
+	    scalefac->s[sfb][b] = s >> 1;
+	}
     }
     cod_info->scalefac_scale = 1;
 }
@@ -730,11 +786,8 @@ static int
 outer_loop (
    lame_global_flags *gfp,
           gr_info        * const cod_info,
-    const FLOAT8                 xr[576],   /* magnitudes of spectral values */
     const III_psy_xmin   * const l3_xmin,   /* allowed distortion of the scalefactor */
-          III_scalefac_t * const scalefac,  /* scalefactors */
           FLOAT8                 xrpow[576], /* coloured magnitudes of spectral values */
-          int                    l3enc[576], /* vector of quantized values ix(0..575) */
     const int                    ch, 
     const int                    targ_bits )  /* maximum allowed bits */
 {
@@ -752,6 +805,7 @@ outer_loop (
     int real_bits;
     int better;
     int over;
+    III_scalefac_t * const scalefac = &cod_info->scalefac;
 
     int copy = 0;
     int age = 0;
@@ -763,8 +817,9 @@ outer_loop (
     
     best_noise_info.over_count = 100;
 
-    bits_found = bin_search_StepSize (gfc, cod_info, targ_bits, 
-                                      gfc->OldValue[ch], xrpow, l3_enc_w);
+    real_bits = bits_found =
+      bin_search_StepSize (gfc, cod_info, targ_bits, 
+			   gfc->OldValue[ch], xrpow, l3_enc_w);
     gfc->OldValue[ch] = cod_info->global_gain;
 
     /* BEGIN MAIN LOOP */
@@ -792,8 +847,6 @@ outer_loop (
                 cod_info->global_gain++;
                 real_bits = inner_loop (gfc, cod_info, huff_bits, xrpow, 
                                         l3_enc_w);
-            } else {
-                real_bits = bits_found;
             }
         } else {
             real_bits = inner_loop (gfc, cod_info, huff_bits, xrpow,
@@ -805,17 +858,16 @@ outer_loop (
         /* compute the distortion in this quantization */
         if (gfc->noise_shaping) 
             /* coefficients and thresholds both l/r (or both mid/side) */
-            over = calc_noise (gfc, xr, l3_enc_w, cod_info, l3_xmin, 
+            over = calc_noise (gfc, l3_enc_w, cod_info, l3_xmin, 
                                scalefac, &distort, &noise_info);
         else {
             /* fast mode, no noise shaping, we are ready */
             best_noise_info = noise_info;
             copy = 0;
-            memcpy(l3enc, l3_enc_w, sizeof(int)*576);
+            memcpy(cod_info->l3_enc, l3_enc_w, sizeof(int)*576);
             break;
         }
 
-
         /* check if this quantization is better
          * than our saved quantization */
         if (iteration == 1) /* the first iteration is always better */
@@ -829,7 +881,7 @@ outer_loop (
         if (better) {
             copy = 0;
             best_noise_info = noise_info;
-            memcpy(l3enc, l3_enc_w, sizeof(int)*576);
+            memcpy(cod_info->l3_enc, l3_enc_w, sizeof(int)*576);
             age = 0;
         }
         else
@@ -919,38 +971,31 @@ outer_loop (
 
 static void 
 iteration_finish (
-    lame_internal_flags *gfc,
-    FLOAT8          xr      [2][2][576],
-    int             l3_enc  [2][2][576],
-    III_scalefac_t  scalefac[2][2],
+    lame_t gfp,
     const int       mean_bits )
 {
+    lame_internal_flags *gfc = gfp->internal_flags;
     III_side_info_t *l3_side = &gfc->l3_side;
-    int gr, ch, i;
+    int gr, ch;
     
     for (gr = 0; gr < gfc->mode_gr; gr++) {
         for (ch = 0; ch < gfc->channels_out; ch++) {
-            gr_info *cod_info = &l3_side->gr[gr].ch[ch].tt;
+            gr_info *cod_info = &l3_side->tt[gr][ch];
 
             /*  try some better scalefac storage
              */
-            best_scalefac_store (gfc, gr, ch, l3_enc, l3_side, scalefac);
+            best_scalefac_store (gfc, gr, ch, l3_side);
             
             /*  best huffman_divide may save some bits too
              */
-            if (gfc->use_best_huffman == 1) 
-                best_huffman_divide (gfc, cod_info, l3_enc[gr][ch]);
+            if (gfc->use_best_huffman == 1 && 
+                gfp->VBR != vbr_mt && gfp->VBR != vbr_mtrh)
+                best_huffman_divide (gfc, cod_info);
             
             /*  update reservoir status after FINAL quantization/bitrate
              */
             ResvAdjust (gfc, cod_info, l3_side, mean_bits);
-      
-            /*  set the sign of l3_enc from the sign of xr
-             */
-            for (i = 0; i < 576; i++) {
-                if (xr[gr][ch][i] < 0) l3_enc[gr][ch][i] *= -1; 
-            }
-        } /* for ch */
+	} /* for ch */
     }    /* for gr */
     
     ResvFrameEnd (gfc, l3_side, mean_bits);
@@ -970,20 +1015,15 @@ static void
 VBR_encode_granule (
           lame_global_flags *gfp,
           gr_info        * const cod_info,
-          FLOAT8                 xr[576],     /* magnitudes of spectral values */
     const III_psy_xmin   * const l3_xmin,     /* allowed distortion of the scalefactor */
-          III_scalefac_t * const scalefac,    /* scalefactors */
           FLOAT8                 xrpow[576],  /* coloured magnitudes of spectral values */
-          int                    l3_enc[576], /* vector of quantized values ix(0..575) */
     const int                    ch, 
           int                    min_bits, 
           int                    max_bits )
 {
     lame_internal_flags *gfc=gfp->internal_flags;
     gr_info         bst_cod_info;
-    III_scalefac_t  bst_scalefac;
     FLOAT8          bst_xrpow [576]; 
-    int             bst_l3_enc[576];
     int Max_bits  = max_bits;
     int real_bits = max_bits+1;
     int this_bits = (max_bits+min_bits)/2;
@@ -1004,8 +1044,8 @@ VBR_encode_granule (
         else
             gfc->sfb21_extra = sfb21_extra;
 
-        over = outer_loop ( gfp, cod_info, xr, l3_xmin, scalefac,
-                            xrpow, l3_enc, ch, this_bits );
+        over = outer_loop ( gfp, cod_info, l3_xmin,
+                            xrpow, ch, this_bits );
 
         /*  is quantization as good as we are looking for ?
          *  in this case: is no scalefactor band distorted?
@@ -1020,9 +1060,7 @@ VBR_encode_granule (
             /*  store best quantization so far
              */
             bst_cod_info = *cod_info;
-            bst_scalefac = *scalefac;
             memcpy(bst_xrpow, xrpow, sizeof(FLOAT8)*576);
-            memcpy(bst_l3_enc, l3_enc, sizeof(int)*576);
 
             /*  try with fewer bits
              */
@@ -1042,7 +1080,6 @@ VBR_encode_granule (
                 /*  start again with best quantization so far
                  */
                 *cod_info = bst_cod_info;
-                *scalefac = bst_scalefac;
                 memcpy(xrpow, bst_xrpow, sizeof(FLOAT8)*576);
             }
         }
@@ -1055,7 +1092,7 @@ VBR_encode_granule (
      *  found=2 => we restored a good one and have now l3_enc to restore too
      */
     if (found==2) {
-        memcpy(l3_enc, bst_l3_enc, sizeof(int)*576);
+        memcpy(cod_info->l3_enc, bst_cod_info.l3_enc, sizeof(int)*576);
     }
     assert(cod_info->part2_3_length <= Max_bits);
 
@@ -1206,7 +1243,6 @@ VBR_prepare (
           lame_global_flags *gfp,
           FLOAT8          pe            [2][2],
           FLOAT8          ms_ener_ratio [2], 
-          FLOAT8          xr            [2][2][576],
           III_psy_ratio   ratio         [2][2], 
           III_psy_xmin    l3_xmin       [2][2],
           int             frameBits     [16],
@@ -1226,19 +1262,18 @@ VBR_prepare (
   
     gfc->bitrate_index = gfc->VBR_max_bitrate;
     getframebits (gfp, &bpf, &avg);
-    bpf = ResvFrameBegin (gfp, &gfc->l3_side, avg, bpf );
-    avg = (bpf - 8*gfc->sideinfo_len) / gfc->mode_gr;
-
+    avg = ResvFrameBegin (gfp, &gfc->l3_side, avg, bpf ) / gfc->mode_gr;
+    
     get_framebits (gfp, analog_mean_bits, min_mean_bits, frameBits);
     
     for (gr = 0; gr < gfc->mode_gr; gr++) {
         mxb = on_pe (gfp, pe, &gfc->l3_side, max_bits[gr], avg, gr);
         if (gfc->mode_ext == MPG_MD_MS_LR) {
-            ms_convert (xr[gr], xr[gr]); 
+            ms_convert (&gfc->l3_side, gr);
             reduce_side (max_bits[gr], ms_ener_ratio[gr], avg, mxb);
         }
         for (ch = 0; ch < gfc->channels_out; ++ch) {
-            gr_info *cod_info = &gfc->l3_side.gr[gr].ch[ch].tt;
+            gr_info *cod_info = &gfc->l3_side.tt[gr][ch];
       
             if (gfc->nsPsy.use && gfp->VBR == vbr_rh) {
             if (cod_info->block_type == NORM_TYPE) 
@@ -1249,7 +1284,8 @@ VBR_prepare (
             masking_lower_db   = gfc->VBR->mask_adjust - adjust; 
             gfc->masking_lower = pow (10.0, masking_lower_db * 0.1);
       
-            bands[gr][ch] = calc_xmin (gfp, xr[gr][ch], ratio[gr]+ch, 
+            init_outer_loop(gfc, cod_info);
+	    bands[gr][ch] = calc_xmin (gfp, &ratio[gr][ch], 
                                        cod_info, l3_xmin[gr]+ch);
             if (bands[gr][ch]) 
                 analog_silence = 0;
@@ -1280,8 +1316,8 @@ VBR_prepare (
 }
  
  
-inline
-void bitpressure_strategy1(
+static void 
+bitpressure_strategy(
     lame_internal_flags * gfc,
     III_psy_xmin l3_xmin[2][2],
     int min_bits[2][2],  
@@ -1290,7 +1326,7 @@ void bitpressure_strategy1(
     int gr, ch, sfb;
     for (gr = 0; gr < gfc->mode_gr; gr++) {
         for (ch = 0; ch < gfc->channels_out; ch++) {
-            if (gfc->l3_side.gr[gr].ch[ch].tt.block_type == SHORT_TYPE) {
+            if (gfc->l3_side.tt[gr][ch].block_type == SHORT_TYPE) {
                 for (sfb = 0; sfb < SBMAX_s; sfb++) {
                     l3_xmin[gr][ch].s[sfb][0] *= 1.+.029*sfb*sfb/SBMAX_s/SBMAX_s;
                     l3_xmin[gr][ch].s[sfb][1] *= 1.+.029*sfb*sfb/SBMAX_s/SBMAX_s;
@@ -1306,23 +1342,6 @@ void bitpressure_strategy1(
     }
 }
 
-inline
-void bitpressure_strategy2( 
-    lame_internal_flags * gfc,
-    int bpf, int used, int save_bits[2][2],
-    int min_bits[2][2], int max_bits[2][2] )  
-{
-    int gr, ch;
-    for (gr = 0; gr < gfc->mode_gr; gr++) {
-        for (ch = 0; ch < gfc->channels_out; ch++) {
-            max_bits[gr][ch]  = save_bits[gr][ch];
-            max_bits[gr][ch] *= bpf;
-            max_bits[gr][ch] /= used;
-            max_bits[gr][ch]  = Max(min_bits[gr][ch],max_bits[gr][ch]);
-        }
-    }
-}
-
 /************************************************************************
  *
  *      VBR_iteration_loop()   
@@ -1339,11 +1358,8 @@ void
 VBR_iteration_loop (
     lame_global_flags *gfp,
     FLOAT8             pe           [2][2],
-    FLOAT8             ms_ener_ratio[2], 
-    FLOAT8             xr           [2][2][576],
-    III_psy_ratio      ratio        [2][2], 
-    int                l3_enc       [2][2][576],
-    III_scalefac_t     scalefac     [2][2] )
+    FLOAT8             ms_ener_ratio[2],
+    III_psy_ratio ratio[2][2])
 {
     lame_internal_flags *gfc=gfp->internal_flags;
     III_psy_xmin l3_xmin[2][2];
@@ -1362,7 +1378,7 @@ VBR_iteration_loop (
     gr_info             *cod_info;
     III_side_info_t     *l3_side  = &gfc->l3_side;
 
-    analog_silence = VBR_prepare (gfp, pe, ms_ener_ratio, xr, ratio, 
+    analog_silence = VBR_prepare (gfp, pe, ms_ener_ratio, ratio, 
                                   l3_xmin, frameBits, &analog_mean_bits,
                                   &min_mean_bits, min_bits, max_bits, bands);
 
@@ -1378,32 +1394,28 @@ VBR_iteration_loop (
     for (gr = 0; gr < gfc->mode_gr; gr++) {
         for (ch = 0; ch < gfc->channels_out; ch++) {
             int ret; 
-            cod_info = &l3_side->gr[gr].ch[ch].tt;
+            cod_info = &l3_side->tt[gr][ch];
       
             /*  init_outer_loop sets up cod_info, scalefac and xrpow 
              */
-            ret = init_outer_loop(gfc, cod_info, &scalefac[gr][ch],
-				  xr[gr][ch], xrpow);
+            ret = init_xrpow(gfc, cod_info, xrpow);
             if (ret == 0 || max_bits[gr][ch] == 0) {
                 /*  xr contains no energy 
                  *  l3_enc, our encoding data, will be quantized to zero
                  */
-                memset(l3_enc[gr][ch], 0, sizeof(int)*576);
                 save_bits[gr][ch] = 0;
                 continue; /* with next channel */
             }
       
             if (gfp->VBR == vbr_mtrh) {
-                ret = VBR_noise_shaping2 (gfp, xr[gr][ch], xrpow, l3_enc[gr][ch],  
-                                        min_bits[gr][ch], max_bits[gr][ch], 
-                                        &scalefac[gr][ch],
-                                        &l3_xmin[gr][ch], gr, ch );
+                ret = VBR_noise_shaping (gfc, xrpow,
+					 min_bits[gr][ch], max_bits[gr][ch], 
+					 &l3_xmin[gr][ch], gr, ch );
                 if (ret < 0)
                     cod_info->part2_3_length = 100000;
             } 
             else
-                VBR_encode_granule (gfp, cod_info, xr[gr][ch], &l3_xmin[gr][ch],
-                                    &scalefac[gr][ch], xrpow, l3_enc[gr][ch],
+	        VBR_encode_granule (gfp, cod_info, &l3_xmin[gr][ch], xrpow,
                                     ch, min_bits[gr][ch], max_bits[gr][ch] );
 
             used_bits += cod_info->part2_3_length;
@@ -1425,25 +1437,17 @@ VBR_iteration_loop (
     for( ; gfc->bitrate_index < gfc->VBR_max_bitrate; gfc->bitrate_index++) {
         if (used_bits <= frameBits[gfc->bitrate_index]) break; 
     }
-
     getframebits (gfp, &bitsPerFrame, &mean_bits);
     bits = ResvFrameBegin (gfp, l3_side, mean_bits, bitsPerFrame);
     
     if (used_bits <= bits) break;
 
-    switch ( gfc -> VBR -> bitpressure ) {
-    default:
-    case  1:    bitpressure_strategy1( gfc, l3_xmin, min_bits, max_bits );
-                break;
-    case  2:    bitpressure_strategy2( gfc, frameBits[gfc->bitrate_index], 
-                               used_bits2, save_bits, min_bits, max_bits );
-                break;
-    }
+    bitpressure_strategy( gfc, l3_xmin, min_bits, max_bits );
 
     }   /* breaks adjusted */
     /*--------------------------------------*/
-    
-    iteration_finish (gfc, xr, l3_enc, scalefac, mean_bits);
+
+    iteration_finish (gfp, mean_bits);
 }
 
 
@@ -1486,7 +1490,7 @@ calc_target_bits (
     mean_bits  = gfp->VBR_mean_bitrate_kbps * gfp->framesize * 1000;
     mean_bits /= gfp->out_samplerate;
     mean_bits -= gfc->sideinfo_len*8;
-    mean_bits /= gfc->mode_gr;
+    mean_bits /= (gfc->mode_gr*gfc->channels_out);
 
     /*
         res_factor is the percentage of the target bitrate that should
@@ -1515,22 +1519,22 @@ calc_target_bits (
 
     for (gr = 0; gr < gfc->mode_gr; gr++) {
         for (ch = 0; ch < gfc->channels_out; ch++) {
-            targ_bits[gr][ch] = res_factor * (mean_bits / gfc->channels_out);
+            targ_bits[gr][ch] = res_factor * mean_bits;
             
             if (pe[gr][ch] > 700) {
                 int add_bits = (pe[gr][ch] - 700) / 1.4;
   
-                gr_info *cod_info = &l3_side->gr[gr].ch[ch].tt;
-                targ_bits[gr][ch] = res_factor * (mean_bits / gfc->channels_out);
+                gr_info *cod_info = &l3_side->tt[gr][ch];
+                targ_bits[gr][ch] = res_factor * mean_bits;
  
                 /* short blocks use a little extra, no matter what the pe */
                 if (cod_info->block_type == SHORT_TYPE) {
-                    if (add_bits < mean_bits/4) 
-                        add_bits = mean_bits/4; 
+                    if (add_bits < mean_bits/2)
+                        add_bits = mean_bits/2;
                 }
                 /* at most increase bits by 1.5*average */
-                if (add_bits > mean_bits*3/4)
-                    add_bits = mean_bits*3/4;
+                if (add_bits > mean_bits*3/2)
+                    add_bits = mean_bits*3/2;
                 else
                 if (add_bits < 0) 
                     add_bits = 0;
@@ -1542,7 +1546,7 @@ calc_target_bits (
     
     if (gfc->mode_ext == MPG_MD_MS_LR) 
         for (gr = 0; gr < gfc->mode_gr; gr++) {
-            reduce_side (targ_bits[gr], ms_ener_ratio[gr], mean_bits,
+            reduce_side (targ_bits[gr], ms_ener_ratio[gr], mean_bits*gfc->channels_out,
 			 MAX_BITS);
         }
 
@@ -1589,17 +1593,14 @@ ABR_iteration_loop(
     lame_global_flags *gfp,
     FLOAT8             pe           [2][2],
     FLOAT8             ms_ener_ratio[2], 
-    FLOAT8             xr           [2][2][576],
-    III_psy_ratio      ratio        [2][2], 
-    int                l3_enc       [2][2][576],
-    III_scalefac_t     scalefac     [2][2] )
+    III_psy_ratio      ratio        [2][2])
 {
     lame_internal_flags *gfc=gfp->internal_flags;
     III_psy_xmin l3_xmin;
     FLOAT8    xrpow[576];
     int       targ_bits[2][2];
     int       bitsPerFrame, mean_bits, totbits, max_frame_bits;
-    int       ch, gr, ath_over, ret;
+    int       ch, gr, ath_over;
     int       analog_silence_bits;
     gr_info             *cod_info;
     III_side_info_t     *l3_side  = &gfc->l3_side;
@@ -1613,34 +1614,25 @@ ABR_iteration_loop(
     for (gr = 0; gr < gfc->mode_gr; gr++) {
 
         if (gfc->mode_ext == MPG_MD_MS_LR) 
-            ms_convert (xr[gr], xr[gr]);
+            ms_convert (&gfc->l3_side, gr);
 
         for (ch = 0; ch < gfc->channels_out; ch++) {
-            cod_info = &l3_side->gr[gr].ch[ch].tt;
+            cod_info = &l3_side->tt[gr][ch];
 
             /*  cod_info, scalefac and xrpow get initialized in init_outer_loop
              */
-            ret = init_outer_loop(gfc, cod_info, &scalefac[gr][ch],
-				  xr[gr][ch], xrpow);
-            if (ret == 0) {
-                /*  xr contains no energy 
-                 *  l3_enc, our encoding data, will be quantized to zero
-                 */
-                memset(l3_enc[gr][ch], 0, sizeof(int)*576);
-            } 
-            else {
+            init_outer_loop(gfc, cod_info);
+            if (init_xrpow(gfc, cod_info, xrpow)) {
                 /*  xr contains energy we will have to encode 
                  *  calculate the masking abilities
                  *  find some good quantization in outer_loop 
                  */
-                ath_over = calc_xmin (gfp, xr[gr][ch], &ratio[gr][ch],
-                                      cod_info, &l3_xmin);
+                ath_over = calc_xmin (gfp, &ratio[gr][ch], cod_info, &l3_xmin);
                 if (0 == ath_over) /* analog silence */
                     targ_bits[gr][ch] = analog_silence_bits;
 
-                outer_loop (gfp, cod_info, xr[gr][ch], &l3_xmin,
-                            &scalefac[gr][ch], xrpow, l3_enc[gr][ch],
-                            ch, targ_bits[gr][ch]);
+                outer_loop (gfp, cod_info, &l3_xmin,
+                            xrpow, ch, targ_bits[gr][ch]);
             }
 
             totbits += cod_info->part2_3_length;
@@ -1658,7 +1650,7 @@ ABR_iteration_loop(
     }
     assert (gfc->bitrate_index <= gfc->VBR_max_bitrate);
 
-    iteration_finish (gfc, xr, l3_enc, scalefac, mean_bits);
+    iteration_finish (gfp, mean_bits);
 }
 
 
@@ -1681,18 +1673,15 @@ iteration_loop(
     lame_global_flags *gfp, 
     FLOAT8             pe           [2][2],
     FLOAT8             ms_ener_ratio[2],  
-    FLOAT8             xr           [2][2][576],
-    III_psy_ratio      ratio        [2][2],  
-    int                l3_enc       [2][2][576],
-    III_scalefac_t     scalefac     [2][2] )
+    III_psy_ratio      ratio        [2][2])
 {
     lame_internal_flags *gfc=gfp->internal_flags;
-    III_psy_xmin l3_xmin[2];
+    III_psy_xmin l3_xmin;
     FLOAT8 xrpow[576];
     int    targ_bits[2];
     int    bitsPerFrame;
     int    mean_bits, max_bits;
-    int    gr, ch, i;
+    int    gr, ch;
     III_side_info_t     *l3_side = &gfc->l3_side;
     gr_info             *cod_info;
 
@@ -1707,43 +1696,34 @@ iteration_loop(
         max_bits = on_pe (gfp, pe, l3_side, targ_bits, mean_bits, gr);
         
         if (gfc->mode_ext == MPG_MD_MS_LR) {
-            ms_convert (xr[gr], xr[gr]);
+            ms_convert (&gfc->l3_side, gr);
             reduce_side (targ_bits, ms_ener_ratio[gr], mean_bits, max_bits);
         }
         
         for (ch=0 ; ch < gfc->channels_out ; ch ++) {
-            cod_info = &l3_side->gr[gr].ch[ch].tt; 
+            cod_info = &l3_side->tt[gr][ch]; 
 
             /*  init_outer_loop sets up cod_info, scalefac and xrpow 
              */
-            i = init_outer_loop(gfc, cod_info, &scalefac[gr][ch],
-				xr[gr][ch], xrpow);
-            if (i == 0) {
-                /*  xr contains no energy, l3_enc will be quantized to zero
-                 */
-                memset(l3_enc[gr][ch], 0, sizeof(int)*576);
-            }
-            else {
+            init_outer_loop(gfc, cod_info);
+            if (init_xrpow(gfc, cod_info, xrpow)) {
                 /*  xr contains energy we will have to encode 
                  *  calculate the masking abilities
                  *  find some good quantization in outer_loop 
                  */
-                calc_xmin (gfp, xr[gr][ch], &ratio[gr][ch], cod_info, 
-                           &l3_xmin[ch]);
-                outer_loop (gfp, cod_info, xr[gr][ch], &l3_xmin[ch], 
-                            &scalefac[gr][ch], xrpow, l3_enc[gr][ch],
-                            ch, targ_bits[ch]);
+                calc_xmin (gfp, &ratio[gr][ch], cod_info, &l3_xmin);
+                outer_loop (gfp, cod_info, &l3_xmin, xrpow, ch, targ_bits[ch]);
             }
             assert (cod_info->part2_3_length <= MAX_BITS);
 
             /*  try some better scalefac storage
              */
-            best_scalefac_store (gfc, gr, ch, l3_enc, l3_side, scalefac);
+            best_scalefac_store (gfc, gr, ch, l3_side);
             
             /*  best huffman_divide may save some bits too
              */
             if (gfc->use_best_huffman == 1) 
-                best_huffman_divide (gfc, cod_info, l3_enc[gr][ch]);
+                best_huffman_divide (gfc, cod_info);
             
             /*  update reservoir status after FINAL quantization/bitrate
              */
@@ -1751,11 +1731,6 @@ iteration_loop(
 #ifndef NORES_TEST
             ResvAdjust (gfc, cod_info, l3_side, mean_bits);
 #endif      
-            /*  set the sign of l3_enc from the sign of xr
-             */
-            for (i = 0; i < 576; i++) {
-                if (xr[gr][ch][i] < 0) l3_enc[gr][ch][i] *= -1; 
-            }
         } /* for ch */
     }    /* for gr */
     
@@ -1765,7 +1740,7 @@ iteration_loop(
        Requires using the --nores.  This is useful for testing only */
     for (gr = 0; gr < gfc->mode_gr; gr++) {
         for (ch =  0; ch < gfc->channels_out; ch++) {
-            cod_info = &l3_side->gr[gr].ch[ch].tt;
+            cod_info = &l3_side->tt[gr][ch];
             ResvAdjust (gfc, cod_info, l3_side, mean_bits);
         }
     }
