@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: quantize_pvt.c,v 1.70 2001/11/25 15:24:08 takehiro Exp $ */
+/* $Id: quantize_pvt.c,v 1.93 2002/09/21 00:31:24 markt Exp $ */
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -303,22 +303,6 @@ void compute_ath( lame_global_flags *gfp, FLOAT8 ATH_l[], FLOAT8 ATH_s[] )
 
 
 
-/* convert from L/R <-> Mid/Side, src == dst allowed */
-void ms_convert(FLOAT8 dst[2][576], FLOAT8 src[2][576])
-{
-    FLOAT8 l;
-    FLOAT8 r;
-    int i;
-    for (i = 0; i < 576; ++i) {
-        l = src[0][i];
-        r = src[1][i];
-        dst[0][i] = (l+r) * (FLOAT8)(SQRT2*0.5);
-        dst[1][i] = (l-r) * (FLOAT8)(SQRT2*0.5);
-    }
-}
-
-
-
 /************************************************************************
  * allocate bits among 2 channels based on PE
  * mt 6/99
@@ -337,6 +321,7 @@ int on_pe( lame_global_flags *gfp, FLOAT8 pe[][2], III_side_info_t *l3_side,
     /* allocate targ_bits for granule */
     ResvMaxBits( gfp, mean_bits, &tbits, &extra_bits );
     max_bits = tbits + extra_bits;
+    mean_bits /= gfc->channels_out;
     if (max_bits > MAX_BITS) /* hard limit per granule */
         max_bits = MAX_BITS;
     
@@ -344,7 +329,7 @@ int on_pe( lame_global_flags *gfp, FLOAT8 pe[][2], III_side_info_t *l3_side,
         /******************************************************************
          * allocate bits for each channel 
          ******************************************************************/
-        cod_info = &l3_side->gr[gr].ch[ch].tt;
+        cod_info = &l3_side->tt[gr][ch];
     
         targ_bits[ch] = Min( MAX_BITS, tbits/gfc->channels_out );
     
@@ -355,14 +340,14 @@ int on_pe( lame_global_flags *gfp, FLOAT8 pe[][2], III_side_info_t *l3_side,
             add_bits[ch] = (pe[gr][ch]-750) / 1.4;
             /* short blocks us a little extra, no matter what the pe */
             if ( cod_info->block_type == SHORT_TYPE ) {
-	        if (add_bits[ch] < mean_bits/4) 
-                    add_bits[ch] = mean_bits/4;
+	        if (add_bits[ch] < mean_bits/2) 
+                    add_bits[ch] = mean_bits/2;
             }
         }
 
         /* at most increase bits by 1.5*average */
-        if (add_bits[ch] > mean_bits*3/4) 
-            add_bits[ch] = mean_bits*3/4;
+        if (add_bits[ch] > mean_bits*3/2)
+            add_bits[ch] = mean_bits*3/2;
         if (add_bits[ch] < 0) 
             add_bits[ch] = 0;
 
@@ -475,116 +460,104 @@ FLOAT8 athAdjust( FLOAT8 a, FLOAT8 x, FLOAT8 athFloor )
 */
 int calc_xmin( 
         lame_global_flags *gfp,
-        const FLOAT8                xr [576],
         const III_psy_ratio * const ratio,
 	const gr_info       * const cod_info, 
               III_psy_xmin  * const l3_xmin ) 
 {
     lame_internal_flags *gfc = gfp->internal_flags;
-    int sfb,j,start, end, bw,l, b, ath_over=0;
-    FLOAT8 en0, xmin, ener, tmpATH;
+    int sfb, j=0, ath_over=0;
+    FLOAT8 xmin, tmpATH;
     ATH_t * ATH = gfc->ATH;
+    const FLOAT8 *xr = cod_info->xr;
 
-    if (cod_info->block_type == SHORT_TYPE) {
+    for (sfb = 0; sfb < cod_info->psy_lmax; sfb++) {
+	FLOAT en0 = 0.0;
+	int width, l;
+	if ( gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh )
+	    tmpATH = athAdjust( ATH->adjust, ATH->l[sfb], ATH->floor );
+	else
+	    tmpATH = ATH->adjust * ATH->l[sfb];
 
-        for ( j = 0, sfb = 0; sfb < SBMAX_s; sfb++ ) {
-            if ( gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh )
-                tmpATH = athAdjust( ATH->adjust, ATH->s[sfb], ATH->floor );
-            else
-                tmpATH = ATH->adjust * ATH->s[sfb];
-            start = gfc->scalefac_band.s[ sfb ];
-            end   = gfc->scalefac_band.s[ sfb + 1 ];
-            bw = end - start;
+	width = gfc->scalefac_band.l[sfb+1] - gfc->scalefac_band.l[sfb];
+	l = width;
+	do {
+	    en0 += xr[j] * xr[j];
+	    j++;
+	} while (--l > 0);
 
-            for ( b = 0; b < 3; b++ ) {
-                for (en0 = 0.0, l = start; l < end; l++) {
-                    ener = xr[j++];
-                    ener = ener * ener;
-                    en0 += ener;
-                }
-                en0 /= bw;
-      
-                if (gfp->ATHonly || gfp->ATHshort) {
-                    xmin = tmpATH;
-                } 
-                else {
-                    xmin = ratio->en.s[sfb][b];
-                    if (xmin > 0.0)
-                        xmin = en0 * ratio->thm.s[sfb][b] * gfc->masking_lower / xmin;
-                    if (xmin < tmpATH) 
-                        xmin = tmpATH;
-                }
-                xmin *= bw;
+	/* why is it different from short blocks <?> */
+	if ( !gfc->nsPsy.use ) en0 /= width;   
 
-                if (gfc->nsPsy.use) {
-                    if      (sfb <=  5) xmin *= gfc->nsPsy.bass;
-                    else if (sfb <= 10) xmin *= gfc->nsPsy.alto;
-                    else                xmin *= gfc->nsPsy.treble;
-                    if ((gfp->VBR == vbr_off || gfp->VBR == vbr_abr) && gfp->quality <= 1)
-                        xmin *= 0.001;
-                }
-                l3_xmin->s[sfb][b] = xmin;
-                if (en0 > tmpATH) ath_over++;
-            }   /* b */
+	xmin = tmpATH;
+	if (!gfp->ATHonly) {
+	    xmin = ratio->en.l[sfb];
+	    if (xmin > 0.0)
+		xmin = en0 * ratio->thm.l[sfb] * gfc->masking_lower / xmin;
+	    if (xmin < tmpATH) 
+		xmin = tmpATH;
+	}
+	/* why is it different from short blocks <?> */
+	if ( !gfc->nsPsy.use ) {
+	    xmin *= width;
+	}
+	else {
+	    if      (sfb <=  6) xmin *= gfc->nsPsy.bass;
+	    else if (sfb <= 13) xmin *= gfc->nsPsy.alto;
+	    else if (sfb <= 20) xmin *= gfc->nsPsy.treble;
+	    else                xmin *= gfc->nsPsy.sfb21;
+	    if ((gfp->VBR == vbr_off || gfp->VBR == vbr_abr) && gfp->quality <= 1)
+		xmin *= 0.001;
+	}
+	l3_xmin->l[sfb] = xmin;
+	if (en0 > tmpATH) ath_over++;
+    }   /* end of long block loop */
+
+    for (sfb = cod_info->sfb_smin; sfb < cod_info->psy_smax; sfb++) {
+	int width, b;
+	if ( gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh )
+	    tmpATH = athAdjust( ATH->adjust, ATH->s[sfb], ATH->floor );
+	else
+	    tmpATH = ATH->adjust * ATH->s[sfb];
+
+	width = gfc->scalefac_band.s[sfb+1] - gfc->scalefac_band.s[sfb];
+	for ( b = 0; b < 3; b++ ) {
+	    FLOAT en0 = 0.0;
+	    int l = width;
+	    do {
+		en0 += xr[j] * xr[j];
+		j++;
+	    } while (--l > 0);
+	    en0 /= width;
+
+	    xmin = tmpATH;
+	    if (!gfp->ATHonly && !gfp->ATHshort) {
+		xmin = ratio->en.s[sfb][b];
+		if (xmin > 0.0)
+		    xmin = en0 * ratio->thm.s[sfb][b] * gfc->masking_lower / xmin;
+		if (xmin < tmpATH) 
+		    xmin = tmpATH;
+	    }
+	    xmin *= width;
+
+	    if (gfc->nsPsy.use) {
+		if      (sfb <=  5) xmin *= gfc->nsPsy.bass;
+		else if (sfb <= 10) xmin *= gfc->nsPsy.alto;
+		else                xmin *= gfc->nsPsy.treble;
+		if ((gfp->VBR == vbr_off || gfp->VBR == vbr_abr) && gfp->quality <= 1)
+		    xmin *= 0.001;
+	    }
+	    l3_xmin->s[sfb][b] = xmin;
+	    if (en0 > tmpATH) ath_over++;
+	}   /* b */
+	if (gfp->useTemporal) {
+	    for ( b = 1; b < 3; b++ ) {
+		xmin = l3_xmin->s[sfb][b] * (1.0 - gfc->decay)
+		    + l3_xmin->s[sfb][b-1] * gfc->decay;
+		if (l3_xmin->s[sfb][b] < xmin)
+		    l3_xmin->s[sfb][b] = xmin;
+	    }
         }   /* sfb */
-
-        if (gfp->useTemporal) {
-            for (sfb = 0; sfb < SBMAX_s; sfb++ ) {
-                for ( b = 1; b < 3; b++ ) {
-                    xmin = l3_xmin->s[sfb][b] * (1.0 - gfc->decay)
-                         + l3_xmin->s[sfb][b-1] * gfc->decay;
-                    if (l3_xmin->s[sfb][b] < xmin)
-                        l3_xmin->s[sfb][b] = xmin;
-                }
-            }
-        }
-
-    }   /* end of short block case */
-    else {
-        
-        for ( sfb = 0; sfb < SBMAX_l; sfb++ ){
-            if ( gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh )
-                tmpATH = athAdjust( ATH->adjust, ATH->l[sfb], ATH->floor );
-            else
-                tmpATH = ATH->adjust * ATH->l[sfb];
-            start = gfc->scalefac_band.l[ sfb ];
-            end   = gfc->scalefac_band.l[ sfb+1 ];
-            bw = end - start;
-    
-            for (en0 = 0.0, l = start; l < end; l++ ) {
-                ener = xr[l] * xr[l];
-                en0 += ener;
-            }
-            /* why is it different from short blocks <?> */
-            if ( !gfc->nsPsy.use ) en0 /= bw;   
-    
-            if (gfp->ATHonly) {
-                xmin = tmpATH;
-            }
-            else {
-                xmin = ratio->en.l[sfb];
-                if (xmin > 0.0)
-                    xmin = en0 * ratio->thm.l[sfb] * gfc->masking_lower / xmin;
-                if (xmin < tmpATH) 
-                    xmin = tmpATH;
-            }
-            /* why is it different from short blocks <?> */
-            if ( !gfc->nsPsy.use ) {
-                xmin *= bw;
-            }
-            else {
-                if      (sfb <=  6) xmin *= gfc->nsPsy.bass;
-                else if (sfb <= 13) xmin *= gfc->nsPsy.alto;
-                else if (sfb <= 20) xmin *= gfc->nsPsy.treble;
-                else                xmin *= gfc->nsPsy.sfb21;
-                if ((gfp->VBR == vbr_off || gfp->VBR == vbr_abr) && gfp->quality <= 1)
-                    xmin *= 0.001;
-            }
-            l3_xmin->l[sfb] = xmin;
-            if (en0 > tmpATH) ath_over++;
-        }   /* sfb */
-	
-    }   /* end of long block case */
+    }   /* end of short block loop */
 
     return ath_over;
 }
@@ -620,7 +593,6 @@ double penalties ( double noise )
 
 int  calc_noise( 
         const lame_internal_flags           * const gfc,
-        const FLOAT8                    xr [576],
         const int                       ix [576],
         const gr_info           * const cod_info,
         const III_psy_xmin      * const l3_xmin, 
@@ -628,123 +600,89 @@ int  calc_noise(
               III_psy_xmin      * xfsf,
               calc_noise_result * const res )
 {
-  int sfb,start, end, j,l, i, over=0;
-  FLOAT8 sum;
-  
-  int count=0;
-  FLOAT8 noise,noise_db;
-  FLOAT8 over_noise_db = 0;
-  FLOAT8 tot_noise_db  = 0;     /*    0 dB relative to masking */
-  FLOAT8 max_noise  = 1E-20; /* -200 dB relative to masking */
-  double klemm_noise = 1E-37;
+    int sfb, l, i, over=0;
+    FLOAT8 over_noise_db = 0;
+    FLOAT8 tot_noise_db  = 0;     /*    0 dB relative to masking */
+    FLOAT8 max_noise  = 1E-20; /* -200 dB relative to masking */
+    double klemm_noise = 1E-37;
+    int j = 0;
 
-  if (cod_info->block_type == SHORT_TYPE) {
-    int max_index = gfc->sfb21_extra ? SBMAX_s : SBPSY_s;
+    for (sfb = 0; sfb < cod_info->psy_lmax; sfb++) {
+	int s =
+	    cod_info->global_gain
+	    - ((scalefac->l[sfb] + (cod_info->preflag ? pretab[sfb] : 0))
+	       << (cod_info->scalefac_scale + 1));
+	FLOAT8 step;
+	FLOAT8 noise = 0.0;
 
-    for ( j=0, sfb = 0; sfb < max_index; sfb++ ) {
-         start = gfc->scalefac_band.s[ sfb ];
-         end   = gfc->scalefac_band.s[ sfb+1 ];
-         for ( i = 0; i < 3; i++ ) {
-	    FLOAT8 step;
-	    int s;
+        if (s<0) {
+            step = pow(2.0, (double)(s - 210) * 0.25);
+        }else{
+            /* use table lookup.  allegedly faster */
+            step = POW20(s);
+        }
 
-	    s = (scalefac->s[sfb][i] << (cod_info->scalefac_scale + 1))
-		+ cod_info->subblock_gain[i] * 8;
-	    s = cod_info->global_gain - s;
+	l = gfc->scalefac_band.l[sfb+1] - gfc->scalefac_band.l[sfb];
+	do {
+	    FLOAT8 temp = fabs(cod_info->xr[j]) - pow43[ix[j]] * step;
+	    noise += temp * temp;
+	    j++;
+	} while (--l > 0);
+	noise = xfsf->l[sfb] = noise / l3_xmin->l[sfb];
+	max_noise=Max(max_noise,noise);
+	klemm_noise += penalties (noise);
 
-	    assert(s<Q_MAX);
-	    assert(s>=0);
-	    step = POW20(s);
-	    sum = 0.0;
-	    l = start;
+	noise = FAST_LOG10(Max(noise,1E-20));
+	/* multiplying here is adding in dB, but can overflow */
+	//tot_noise *= Max(noise, 1E-20);
+	tot_noise_db += noise;
+
+	if (noise > 0.0) {
+	    over++;
+	    /* multiplying here is adding in dB -but can overflow */
+	    //over_noise *= noise;
+	    over_noise_db += noise;
+	}
+    }
+
+    for (sfb = cod_info->sfb_smin; sfb < cod_info->psy_smax; sfb++) {
+	int width = gfc->scalefac_band.s[sfb+1] - gfc->scalefac_band.s[sfb];
+	for ( i = 0; i < 3; i++ ) {
+	    int s =
+		cod_info->global_gain
+		- (scalefac->s[sfb][i] << (cod_info->scalefac_scale + 1))
+		- cod_info->subblock_gain[i] * 8;
+	    FLOAT8 step = POW20(s);
+	    FLOAT8 noise = 0.0;
+	    l = width;
 	    do {
-	      FLOAT8 temp;
-	      temp = pow43[ix[j]];
-	      temp *= step;
-	      temp -= fabs(xr[j]);
-	      ++j;
-	      sum += temp * temp;
-	      l++;
-	    } while (l < end);
-            noise = xfsf->s[sfb][i]  = sum / l3_xmin->s[sfb][i];
+		FLOAT8 temp;
+		temp = pow43[ix[j]] * step - fabs(cod_info->xr[j]);
+		noise += temp * temp;
+		j++;
+	    } while (--l > 0);
+	    noise = xfsf->s[sfb][i]  = noise / l3_xmin->s[sfb][i];
 
 	    max_noise    = Max(max_noise,noise);
 	    klemm_noise += penalties (noise);
 
-	    noise_db=10*log10(Max(noise,1E-20));
-	    tot_noise_db += noise_db;
+	    noise = FAST_LOG10(Max(noise,1E-20));
+	    tot_noise_db += noise;
 
-            if (noise > 1) {
+	    if (noise > 0.0) {
 		over++;
-		over_noise_db += noise_db;
+		over_noise_db += noise;
 	    }
-	    count++;	    
-        }
-    }
-  }else{ /* cod_info->block_type == SHORT_TYPE */
-    int max_index = gfc->sfb21_extra ? SBMAX_l : SBPSY_l;
-    
-    for ( sfb = 0; sfb < max_index; sfb++ ) {
-	FLOAT8 step;
-	int s = scalefac->l[sfb];
-
-	if (cod_info->preflag)
-	    s += pretab[sfb];
-
-	s = cod_info->global_gain - (s << (cod_info->scalefac_scale + 1));
-	assert(s<Q_MAX);
-	assert(s>=0);
-	step = POW20(s);
-
-	start = gfc->scalefac_band.l[ sfb ];
-        end   = gfc->scalefac_band.l[ sfb+1 ];
-
-	for ( sum = 0.0, l = start; l < end; l++ ) {
-	  FLOAT8 temp;
-	  temp = fabs(xr[l]) - pow43[ix[l]] * step;
-	  sum += temp * temp;
 	}
-	noise = xfsf->l[sfb] = sum / l3_xmin->l[sfb];
-	max_noise=Max(max_noise,noise);
-	klemm_noise += penalties (noise);
-
-	noise_db=10*log10(Max(noise,1E-20));
-	/* multiplying here is adding in dB, but can overflow */
-	//tot_noise *= Max(noise, 1E-20);
-	tot_noise_db += noise_db;
-
-	if (noise > 1) {
-	  over++;
-	  /* multiplying here is adding in dB -but can overflow */
-	  //over_noise *= noise;
-	  over_noise_db += noise_db;
-	}
-
-	count++;
     }
-  } /* cod_info->block_type == SHORT_TYPE */
 
-  /* normalization at this point by "count" is not necessary, since
-   * the values are only used to compare with previous values */
-  res->over_count = over;
+    res->over_count = over;
+    res->tot_noise   = 10.*tot_noise_db;
+    res->over_noise  = 10.*over_noise_db;
+    res->max_noise   = 10.*FAST_LOG10(max_noise);
+    res->klemm_noise = klemm_noise;
 
-  /* convert to db. DO NOT CHANGE THESE */
-  /* tot_noise = is really the average over each sfb of: 
-     [noise(db) - allowed_noise(db)]
-
-     and over_noise is the same average, only over only the
-     bands with noise > allowed_noise.  
-
-  */
-
-  //res->tot_noise   = 10.*log10(Max(1e-20,tot_noise )); 
-  //res->over_noise  = 10.*log10(Max(1e+00,over_noise)); 
-  res->tot_noise   = tot_noise_db;
-  res->over_noise  = over_noise_db;
-  res->max_noise   = 10.*log10(Max(1e-20,max_noise ));
-  res->klemm_noise = 10.*log10(Max(1e-20,klemm_noise));
-
-  return over;
+    return over;
 }
 
 
@@ -768,18 +706,16 @@ int  calc_noise(
  *
  *  Mark Taylor 2000-??-??                
  *
- *  Robert Hegemann: moved noise/distortion calc into it                          
+ *  Robert Hegemann: moved noise/distortion calc into it
  *
  ************************************************************************/
 
 static
 void set_pinfo (
         lame_global_flags *gfp,
-        const gr_info        * const cod_info,
+              gr_info        * const cod_info,
         const III_psy_ratio  * const ratio, 
         const III_scalefac_t * const scalefac,
-        const FLOAT8                 xr[576],
-        const int                    l3_enc[576],        
         const int                    gr,
         const int                    ch )
 {
@@ -794,9 +730,9 @@ void set_pinfo (
     calc_noise_result noise;
     III_psy_xmin xfsf;
 
-    calc_xmin (gfp,xr, ratio, cod_info, &l3_xmin);
-
-    calc_noise (gfc, xr, l3_enc, cod_info, &l3_xmin, scalefac, &xfsf, &noise);
+    calc_xmin (gfp, ratio, cod_info, &l3_xmin);
+    calc_noise (gfc, cod_info->l3_enc, cod_info,
+		&l3_xmin, scalefac, &xfsf, &noise);
 
     if (cod_info->block_type == SHORT_TYPE) {
         for (j=0, sfb = 0; sfb < SBMAX_s; sfb++ )  {
@@ -805,7 +741,7 @@ void set_pinfo (
             bw = end - start;
             for ( i = 0; i < 3; i++ ) {
                 for ( en0 = 0.0, l = start; l < end; l++ ) {
-                    en0 += xr[j] * xr[j];
+                    en0 += cod_info->xr[j] * cod_info->xr[j];
                     ++j;
                 }
                 en0=Max(en0/bw,1e-20);
@@ -878,8 +814,8 @@ void set_pinfo (
             end   = gfc->scalefac_band.l[ sfb+1 ];
             bw = end - start;
             for ( en0 = 0.0, l = start; l < end; l++ ) 
-                en0 += xr[l] * xr[l];
-            if (!gfc->nsPsy.use) en0/=bw;
+                en0 += cod_info->xr[l] * cod_info->xr[l];
+            en0/=bw;
       /*
     DEBUGF("diff  = %f \n",10*log10(Max(ratio[gr][ch].en.l[sfb],1e-20))
                             -(10*log10(en0)+150));
@@ -968,16 +904,12 @@ void set_pinfo (
 
 void set_frame_pinfo( 
         lame_global_flags *gfp,
-        FLOAT8          xr       [2][2][576],
-        III_psy_ratio   ratio    [2][2],  
-        int             l3_enc   [2][2][576],
-        III_scalefac_t  scalefac [2][2] )
+        III_psy_ratio   ratio    [2][2])
 {
     lame_internal_flags *gfc=gfp->internal_flags;
     unsigned int          sfb;
-	int                   ch;
-	int                   gr;
-    int                   act_l3enc[576];
+    int                   ch;
+    int                   gr;
     III_scalefac_t        act_scalefac [2];
     int scsfi[2] = {0,0};
     
@@ -988,11 +920,12 @@ void set_frame_pinfo(
      */
     for (ch = 0; ch < gfc->channels_out; ch ++) {
         for (sfb = 0; sfb < SBMAX_l; sfb ++) {
-            if (scalefac[1][ch].l[sfb] == -1) {/* scfsi */
-                act_scalefac[ch].l[sfb] = scalefac[0][ch].l[sfb];
+            gr_info *cod_info = &gfc->l3_side.tt[0][ch];
+            if (gfc->l3_side.tt[1][ch].scalefac.l[sfb] == -1) {/* scfsi */
+                act_scalefac[ch].l[sfb] = cod_info->scalefac.l[sfb];
                 scsfi[ch] = 1;
             } else {
-                act_scalefac[ch].l[sfb] = scalefac[1][ch].l[sfb];
+                act_scalefac[ch].l[sfb] = gfc->l3_side.tt[1][ch].scalefac.l[sfb];
             }
         }
     }
@@ -1001,22 +934,14 @@ void set_frame_pinfo(
      */
     for (gr = 0; gr < gfc->mode_gr; gr ++) {
         for (ch = 0; ch < gfc->channels_out; ch ++) {
-            int i;
-            gr_info *cod_info = &gfc->l3_side.gr[gr].ch[ch].tt;
+            gr_info *cod_info = &gfc->l3_side.tt[gr][ch];
             
-            /* revert back the sign of l3enc */
-            for ( i = 0; i < 576; i++) {
-                if (xr[gr][ch][i] < 0) 
-                    act_l3enc[i] = -l3_enc[gr][ch][i];
-                else
-                    act_l3enc[i] = +l3_enc[gr][ch][i];
-            }
             if (gr == 1 && scsfi[ch]) 
                 set_pinfo (gfp, cod_info, &ratio[gr][ch], &act_scalefac[ch],
-                        xr[gr][ch], act_l3enc, gr, ch);                    
+			   gr, ch);
             else
-                set_pinfo (gfp, cod_info, &ratio[gr][ch], &scalefac[gr][ch],
-                        xr[gr][ch], act_l3enc, gr, ch);                    
+                set_pinfo (gfp, cod_info, &ratio[gr][ch], &cod_info->scalefac,
+			   gr, ch);
         } /* for ch */
     }    /* for gr */
 }
