@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: set_get.c,v 1.32 2001/12/23 20:19:00 dibrom Exp $ */
+/* $Id: set_get.c,v 1.42 2002/10/16 18:01:10 bouvigne Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -35,6 +35,12 @@
 #endif
 
 
+
+extern int apply_preset(lame_global_flags*  gfp, int preset);
+
+
+
+
 /*
  * input stream description
  */
@@ -472,6 +478,10 @@ lame_set_brate( lame_global_flags*  gfp,
 {
     gfp->brate = brate;
 
+    if (brate >= 320) {
+        gfp->disable_reservoir = 1;
+    }
+
     return 0;
 }
 
@@ -775,6 +785,40 @@ lame_get_exp_nspsytune( const lame_global_flags*  gfp )
 }
 
 
+int
+lame_set_exp_nspsytune2_int( lame_global_flags*  gfp,
+			     int adr,int val)
+{
+  int ret = gfp->exp_nspsytune2.integer[adr];
+
+  gfp->exp_nspsytune2.integer[adr] = val;
+
+  return ret;
+}
+
+
+float
+lame_set_exp_nspsytune2_real( lame_global_flags*  gfp,
+			      int adr,float val)
+{
+  float ret = gfp->exp_nspsytune2.real[adr];
+
+  gfp->exp_nspsytune2.real[adr] = val;
+
+  return ret;
+}
+
+
+void *
+lame_set_exp_nspsytune2_pointer( lame_global_flags*  gfp,
+				 int adr,void *val)
+{
+  void *ret = gfp->exp_nspsytune2.pointer[adr];
+
+  gfp->exp_nspsytune2.pointer[adr] = val;
+
+  return ret;
+}
 
 
 /********************************************************************
@@ -1216,6 +1260,51 @@ lame_get_useTemporal( const lame_global_flags*  gfp )
 }
 
 
+/* Use temporal masking effect */
+int
+lame_set_interChRatio( lame_global_flags*  gfp,
+			float               ratio )
+{
+    /* default = 0.0 (no inter-cahnnel maskin) */
+    if (! (0 <= ratio && ratio <= 1.0))
+        return -1;
+
+    gfp->interChRatio = ratio;
+
+    return 0;
+}
+
+float
+lame_get_interChRatio( const lame_global_flags*  gfp )
+{
+    assert( 0 <= gfp->interChRatio && gfp->interChRatio <= 1.0);
+
+    return gfp->interChRatio;
+}
+
+
+/* Use pseudo substep shaping method */
+int
+lame_set_substep( lame_global_flags*  gfp,
+		  int                 method )
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    /* default = 0.0 (no inter-cahnnel maskin) */
+    if (! (0 <= method && method <= 2))
+        return -1;
+
+    gfc->substep_shaping = method;
+    return 0;
+}
+
+int
+lame_get_substep(const lame_global_flags*  gfp )
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    assert(0 <= gfc->substep_shaping && gfc->substep_shaping <= 2);
+    return gfc->substep_shaping;
+}
+
 /* Disable short blocks. */
 int
 lame_set_no_short_blocks( lame_global_flags*  gfp,
@@ -1533,9 +1622,38 @@ lame_set_preset_notune( lame_global_flags*  gfp, int preset_notune )
 }
 
 
+int
+lame_set_preset( lame_global_flags*  gfp, int preset )
+{
+    gfp->preset = preset;
+    return apply_preset(gfp, preset);
+}
 
 
 
+int 
+lame_set_asm_optimizations( lame_global_flags*  gfp, int optim, int mode)
+{
+    mode = (mode == 1? 1 : 0);
+    switch (optim){
+        case MMX: {
+            gfp->asm_optimizations.mmx = mode;
+            return optim;
+        }
+        case AMD_3DNOW: {
+            gfp->asm_optimizations.amd3dnow = mode;
+            return optim;
+        }
+        case SSE: {
+            gfp->asm_optimizations.sse = mode;
+            return optim;
+        }
+        default: return optim;
+    }
+
+}
+
+
 
 
 
